@@ -20,7 +20,6 @@
 
 #include "prim.h"
 #include "diag.h"
-#include "fbuf.h"
 
 typedef enum {
    // Statement/Line coverage
@@ -221,15 +220,16 @@ typedef enum {
 } merge_mode_t;
 
 cover_data_t *cover_data_init(cover_mask_t mask, int array_limit, int threshold);
+void cover_data_free(cover_data_t *db);
 bool cover_enabled(cover_data_t *data, cover_mask_t mask);
 
-unsigned cover_count_items(cover_data_t *data);
-
-void cover_dump_items(cover_data_t *data, fbuf_t *f, cover_dump_t dt,
-                      const int32_t *counts);
+void cover_dump_items(cover_data_t *data, fbuf_t *f, cover_dump_t dt);
 cover_data_t *cover_read_items(fbuf_t *f, uint32_t pre_mask);
 void cover_merge_items(fbuf_t *f, cover_data_t *data, merge_mode_t mode);
 
+int32_t *cover_get_counters(cover_data_t *db, ident_t name);
+cover_scope_t *cover_get_scope(cover_data_t *db, ident_t name);
+
 //
 // Spec and exclude file handling
 //
@@ -243,6 +243,9 @@ void cover_load_exclude_file(const char *path, cover_data_t *data);
 // Report generation and export
 //
 
+cover_rpt_t *cover_report_new(cover_data_t *db, int item_limit);
+void cover_report_free(cover_rpt_t *rpt);
+
 void cover_report(const char *path, cover_data_t *data, int item_limit);
 void cover_export_cobertura(cover_data_t *data, FILE *f,
                             const char *relative);
@@ -252,10 +255,11 @@ void cover_export_xml(cover_data_t *data, FILE *f, const char *relative);
 // Interface to code generator
 //
 
+cover_scope_t *cover_create_block(cover_data_t *data, ident_t qual,
+                                  cover_scope_t *parent, tree_t inst,
+                                  tree_t unit, ident_t name);
 cover_scope_t *cover_create_scope(cover_data_t *data, cover_scope_t *parent,
                                   tree_t t, ident_t name);
-cover_scope_t *cover_create_instance(cover_data_t *data, cover_scope_t *parent,
-                                     tree_t block, tree_t unit);
 cover_item_t *cover_add_items_for(cover_data_t *data, cover_scope_t *cscope,
                                   object_t *obj, cover_item_kind_t kind);
 
