@@ -302,7 +302,13 @@ static void dump_expr(tree_t t)
             dump_expr(value);
             break;
          case A_OTHERS:
-            print_syntax("#others => ");
+            print_syntax("#others ");
+            if (tree_ranges(a) > 0) {
+               print_syntax("/*");
+               dump_range(tree_range(a, 0));
+               print_syntax("*/ ");
+            }
+            print_syntax("=> ");
             dump_expr(value);
             break;
          case A_SLICE:
@@ -811,9 +817,12 @@ static void dump_view_decl(tree_t t, int indent)
 static void dump_decl(tree_t t, int indent)
 {
    tab(indent);
-   if (tree_kind(t) != T_HIER) dump_address(t);
 
-   switch (tree_kind(t)) {
+   const tree_kind_t kind = tree_kind(t);
+   if (kind != T_HIER)
+      dump_address(t);
+
+   switch (kind) {
    case T_IMPLICIT_SIGNAL:
       print_syntax("/* implicit */ ");
       // Fall-through
@@ -1035,6 +1044,10 @@ static void dump_decl(tree_t t, int indent)
       dump_view_decl(t, 0);
       return;
 
+   case T_VERILOG:
+      vlog_dump(tree_vlog(t), indent);
+      return;
+
    default:
       cannot_dump(t, "decl");
    }
