@@ -34,6 +34,7 @@
 #include "rt/structs.h"
 #include "thread.h"
 #include "type.h"
+#include "vhdl/vhdl-phase.h"
 #include "vlog/vlog-defs.h"
 #include "vlog/vlog-node.h"
 #include "vlog/vlog-phase.h"
@@ -50,7 +51,6 @@ typedef A(tree_t) tree_list_t;
 
 typedef struct _elab_ctx elab_ctx_t;
 typedef struct _generic_list generic_list_t;
-typedef struct _elab_instance elab_instance_t;
 
 typedef struct _elab_ctx {
    const elab_ctx_t *parent;
@@ -73,6 +73,7 @@ typedef struct _elab_ctx {
    rt_model_t       *model;
    rt_scope_t       *scope;
    unsigned          depth;
+   unsigned          errors;
 } elab_ctx_t;
 
 typedef struct {
@@ -94,17 +95,15 @@ typedef struct _generic_list {
 
 typedef struct {
    tree_t           wrap;
-   tree_t           entity;
    vlog_node_t      module;
-   elab_instance_t *instances;
+   ghash_t         *instances;
 } mod_cache_t;
 
-typedef struct _elab_instance {
-   elab_instance_t *next;
-   mod_cache_t     *module;
-   vlog_node_t      body;
-   tree_t           block;
-   tree_t           wrap;
+typedef struct {
+   mod_cache_t *module;
+   vlog_node_t  body;
+   tree_t       block;
+   tree_t       wrap;
 } elab_instance_t;
 
 static void elab_block(tree_t t, const elab_ctx_t *ctx);
@@ -235,6 +234,41 @@ static void elab_subprogram_prefix(tree_t arch, elab_ctx_t *ctx)
    ctx->prefix[1] = tree_ident(tree_primary(arch));
 }
 
+static uint32_t elab_hash_inst(const void *key)
+{
+   vlog_node_t v = (void *)key;
+   assert(vlog_kind(v) == V_INST_LIST);
+
+   uint32_t h = ident_hash(vlog_ident(v));
+
+   const int nparams = vlog_params(v);
+   for (int i = 0; i < nparams; i++)
+      h ^= vlog_hash_node(vlog_param(v, i));
+
+   return h;
+}
+
+static bool elab_cmp_inst(const void *a, const void *b)
+{
+   vlog_node_t va = (vlog_node_t)a, vb = (vlog_node_t)b;
+   assert(vlog_kind(va) == V_INST_LIST);
+   assert(vlog_kind(vb) == V_INST_LIST);
+
+   if (vlog_ident(va) != vlog_ident(vb))
+      return false;
+
+   const int nparams = vlog_params(va);
+   if (vlog_params(vb) != nparams)
+      return false;
+
+   for (int i = 0; i < nparams; i++) {
+      if (!vlog_equal_node(vlog_param(va, i), vlog_param(vb, i)))
+         return false;
+   }
+
+   return true;
+}
+
 static mod_cache_t *elab_cached_module(vlog_node_t mod, const elab_ctx_t *ctx)
 {
    assert(is_top_level(mod));
@@ -244,7 +278,8 @@ static mod_cache_t *elab_cached_module(vlog_node_t mod, const elab_ctx_t *ctx)
       return mc;
 
    mc = xcalloc(sizeof(mod_cache_t));
-   mc->module = mod;
+   mc->module    = mod;
+   mc->instances = ghash_new(4, elab_hash_inst, elab_cmp_inst);
 
    mc->wrap = tree_new(T_VERILOG);
    tree_set_loc(mc->wrap, vlog_loc(mod));
@@ -262,8 +297,12 @@ static elab_instance_t *elab_new_instance(vlog_node_t mod, vlog_node_t inst,
 
    mod_cache_t *mc = elab_cached_module(mod, ctx);
 
-   elab_instance_t *ei = xcalloc(sizeof(elab_instance_t));
-   ei->next   = mc->instances;
+   elab_instance_t *ei = ghash_get(mc->instances, inst);
+   if (ei != NULL) {
+      return ei;
+   }
+
+   ei = xcalloc(sizeof(elab_instance_t));
    ei->module = mc;
    ei->body   = vlog_new_instance(mod, inst, ctx->dotted);
 
@@ -278,8 +317,7 @@ static elab_instance_t *elab_new_instance(vlog_node_t mod, vlog_node_t inst,
 
    vlog_trans(ei->body, ei->block);
 
-   mc->instances = ei;
-
+   ghash_put(mc->instances, inst, ei);
    return ei;
 }
 
@@ -310,6 +348,10 @@ static tree_t elab_to_vhdl(type_t from, type_t to)
         "NVC.VERILOG.TO_VHDL(" T_WIRE_ARRAY ")Y" },
       { VERILOG_WIRE_ARRAY, IEEE_STD_LOGIC_VECTOR,
         "NVC.VERILOG.TO_VHDL(" T_WIRE_ARRAY ")Y" },
+      { VERILOG_LOGIC_ARRAY, IEEE_STD_ULOGIC_VECTOR,
+        "NVC.VERILOG.TO_VHDL(" T_LOGIC_ARRAY ")Y" },
+      { VERILOG_LOGIC_ARRAY, IEEE_STD_LOGIC_VECTOR,
+        "NVC.VERILOG.TO_VHDL(" T_LOGIC_ARRAY ")Y" },
    };
 
    INIT_ONCE({
@@ -363,33 +405,25 @@ static tree_t elab_to_verilog(type_t from, type_t to)
    return NULL;
 }
 
-static tree_t elab_mixed_binding(tree_t comp, mod_cache_t *mc)
+static tree_t elab_mixed_binding(tree_t comp, const elab_instance_t *ei)
 {
    assert(tree_kind(comp) == T_COMPONENT);
 
-   if (mc->entity == NULL) {
-      mc->entity = tree_new(T_ENTITY);
-      tree_set_ident(mc->entity, vlog_ident(mc->module));
-      tree_set_loc(mc->entity, vlog_loc(mc->module));
-
-      vlog_trans(mc->module, mc->entity);
-   }
-
    tree_t bind = tree_new(T_BINDING);
-   tree_set_ident(bind, vlog_ident(mc->module));
+   tree_set_ident(bind, tree_ident(ei->wrap));
    tree_set_loc(bind, tree_loc(comp));
-   tree_set_ref(bind, mc->wrap);
+   tree_set_ref(bind, ei->wrap);
    tree_set_class(bind, C_ENTITY);
 
    const int vhdl_nports = tree_ports(comp);
-   const int vlog_nports = vlog_ports(mc->module);
+   const int vlog_nports = vlog_ports(ei->body);
 
-   bit_mask_t have;
+   LOCAL_BIT_MASK have;
    mask_init(&have, vhdl_nports);
 
    bool have_named = false;
    for (int i = 0; i < vlog_nports; i++) {
-      vlog_node_t ref = vlog_port(mc->module, i);
+      vlog_node_t ref = vlog_port(ei->body, i);
       assert(vlog_kind(ref) == V_REF);
 
       vlog_node_t mport = vlog_ref(ref);
@@ -397,7 +431,7 @@ static tree_t elab_mixed_binding(tree_t comp, mod_cache_t *mc)
 
       ident_t name = vlog_ident2(mport);
 
-      tree_t vport = tree_port(mc->entity, i);
+      tree_t vport = tree_port(ei->block, i);
       assert(tree_ident(vport) == vlog_ident(mport));
 
       tree_t cport = NULL;
@@ -417,10 +451,30 @@ static tree_t elab_mixed_binding(tree_t comp, mod_cache_t *mc)
          return NULL;
       }
 
+      const v_port_kind_t mdir = vlog_subkind(mport);
+      const port_mode_t cdir = tree_subkind(cport);
+
+      if (mdir != V_PORT_INPUT && mdir != V_PORT_OUTPUT) {
+         error_at(tree_loc(cport), "only input and output ports are supported "
+                  "for mixed language instantiation");
+         return NULL;
+      }
+      else if (!(mdir == V_PORT_INPUT && cdir == PORT_IN)
+               && !(mdir == V_PORT_OUTPUT && cdir == PORT_OUT)) {
+         error_at(tree_loc(cport), "VHDL port direction %s does not match "
+                  "corresponding Verilog port '%s' which is declared as %s",
+                  port_mode_str(cdir), istr(vlog_ident(mport)),
+                  mdir == V_PORT_INPUT ? "input" : "output");
+         return NULL;
+      }
+
       type_t btype = tree_type(cport);
       type_t vtype = tree_type(vport);
 
-      if (vlog_subkind(mport) == V_PORT_INPUT) {
+      tree_t conv = tree_new(T_CONV_FUNC);
+      tree_set_loc(conv, tree_loc(cport));
+
+      if (mdir == V_PORT_INPUT) {
          tree_t func = elab_to_verilog(btype, vtype);
          if (func == NULL) {
             error_at(tree_loc(cport), "cannot connect VHDL signal with type "
@@ -429,8 +483,6 @@ static tree_t elab_mixed_binding(tree_t comp, mod_cache_t *mc)
             return NULL;
          }
 
-         tree_t conv = tree_new(T_CONV_FUNC);
-         tree_set_loc(conv, tree_loc(cport));
          tree_set_ref(conv, func);
          tree_set_ident(conv, tree_ident(func));
          tree_set_type(conv, type_result(tree_type(func)));
@@ -450,8 +502,6 @@ static tree_t elab_mixed_binding(tree_t comp, mod_cache_t *mc)
             return NULL;
          }
 
-         tree_t conv = tree_new(T_CONV_FUNC);
-         tree_set_loc(conv, tree_loc(cport));
          tree_set_ref(conv, func);
          tree_set_ident(conv, tree_ident(func));
          tree_set_type(conv, type_result(tree_type(func)));
@@ -467,213 +517,11 @@ static tree_t elab_mixed_binding(tree_t comp, mod_cache_t *mc)
          tree_t p = tree_port(comp, i);
          diag_t *d = diag_new(DIAG_ERROR, tree_loc(p));
          diag_printf(d, "port %s not found in Verilog module %s",
-                     istr(tree_ident(p)), istr(vlog_ident2(mc->module)));
+                     istr(tree_ident(p)), istr(vlog_ident2(ei->body)));
          diag_emit(d);
       }
    }
 
-   mask_free(&have);
-
-   const int vhdl_ngenerics = tree_generics(comp);
-   const int vlog_ndecls = vlog_decls(mc->module);
-
-   mask_init(&have, vhdl_ngenerics);
-
-   for (int i = 0, nth = 0; i < vlog_ndecls; i++) {
-      vlog_node_t d = vlog_decl(mc->module, i);
-      if (vlog_kind(d) != V_PARAM_DECL)
-         continue;
-
-      ident_t name = vlog_ident(d);
-
-      tree_t eg = tree_generic(mc->entity, nth++);
-      assert(tree_ident(eg) == name);
-
-      tree_t cg = NULL;
-      for (int j = 0; j < vhdl_ngenerics; j++) {
-         tree_t gj = tree_generic(comp, j);
-         if (ident_casecmp(tree_ident(gj), name)) {
-            cg = gj;
-            mask_set(&have, j);
-            break;
-         }
-      }
-
-      if (cg == NULL) {
-         error_at(tree_loc(comp), "missing matching VHDL generic declaration "
-                  "for Verilog parameter '%s' in component %s", istr(name),
-                  istr(tree_ident(comp)));
-         return NULL;
-      }
-
-      type_t ctype = tree_type(cg);
-      type_t etype = tree_type(eg);
-      assert(type_eq(etype, std_type(NULL, STD_INTEGER)));
-
-      if (!type_eq(ctype, etype)) {
-         error_at(tree_loc(cg), "generic %s should have type %s to match "
-                  "corresponding Verilog parameter", istr(tree_ident(cg)),
-                  type_pp(etype));
-         return NULL;
-      }
-
-      tree_t map = tree_new(T_PARAM);
-      tree_set_subkind(map, P_POS);
-      tree_set_pos(map, tree_genmaps(bind));
-      tree_set_value(map, make_ref(cg));
-
-      tree_add_genmap(bind, map);
-   }
-
-   for (int i = 0; i < vhdl_ngenerics; i++) {
-      if (!mask_test(&have, i)) {
-         tree_t g = tree_generic(comp, i);
-         diag_t *d = diag_new(DIAG_ERROR, tree_loc(g));
-         diag_printf(d, "generic %s not found in Verilog module %s",
-                     istr(tree_ident(g)), istr(vlog_ident2(mc->module)));
-         diag_emit(d);
-      }
-   }
-
-   mask_free(&have);
-   return bind;
-}
-
-static tree_t elab_verilog_conversion(type_t from, type_t to)
-{
-   static struct {
-      const verilog_type_t from_id;
-      const verilog_type_t to_id;
-      const char *const    func;
-      type_t               from;
-      type_t               to;
-      tree_t               decl;
-   } table[] = {
-      { VERILOG_NET_VALUE, VERILOG_LOGIC,
-        "NVC.VERILOG.TO_LOGIC(" T_NET_VALUE ")" T_LOGIC },
-      { VERILOG_NET_ARRAY, VERILOG_LOGIC_ARRAY,
-        "NVC.VERILOG.TO_LOGIC(" T_NET_ARRAY ")" T_LOGIC_ARRAY },
-      { VERILOG_WIRE_ARRAY, VERILOG_LOGIC_ARRAY,
-        "NVC.VERILOG.TO_LOGIC(" T_WIRE_ARRAY ")" T_LOGIC_ARRAY },
-      { VERILOG_LOGIC, VERILOG_NET_VALUE,
-        "NVC.VERILOG.TO_NET(" T_LOGIC ")" T_NET_VALUE },
-      { VERILOG_LOGIC_ARRAY, VERILOG_NET_ARRAY,
-        "NVC.VERILOG.TO_NET(" T_LOGIC_ARRAY ")" T_NET_ARRAY },
-      { VERILOG_LOGIC_ARRAY, VERILOG_WIRE_ARRAY,
-        "NVC.VERILOG.TO_NET(" T_LOGIC_ARRAY ")" T_WIRE_ARRAY },
-   };
-
-   INIT_ONCE({
-         for (int i = 0; i < ARRAY_LEN(table); i++) {
-            table[i].from = verilog_type(table[i].from_id);
-            table[i].to   = verilog_type(table[i].to_id);
-            table[i].decl = verilog_func(ident_new(table[i].func));
-         }
-      });
-
-   for (int i = 0; i < ARRAY_LEN(table); i++) {
-      if (type_eq(table[i].from, from) && type_eq(table[i].to, to))
-         return table[i].decl;
-   }
-
-   return NULL;
-}
-
-static tree_t elab_verilog_binding(vlog_node_t inst, elab_instance_t *ei,
-                                   const elab_ctx_t *ctx)
-{
-   assert(vlog_kind(inst) == V_MOD_INST);
-
-   tree_t bind = tree_new(T_BINDING);
-   tree_set_ident(bind, vlog_ident(ei->body));
-   tree_set_loc(bind, vlog_loc(inst));
-   tree_set_ref(bind, ei->module->wrap);
-   tree_set_class(bind, C_ENTITY);
-
-   const int nports = vlog_ports(ei->body);
-   const int nparams = vlog_params(inst);
-   const int outports = tree_ports(ctx->out);
-   const int outdecls = tree_decls(ctx->out);
-
-   if (nports != nparams) {
-      error_at(vlog_loc(inst), "expected %d port connections for module %s "
-               "but found %d", nports, istr(vlog_ident2(ei->body)), nparams);
-      return NULL;
-   }
-
-   bool have_named = false;
-   for (int i = 0; i < nports; i++) {
-      vlog_node_t conn = vlog_param(inst, i);
-      assert(vlog_kind(conn) == V_PORT_CONN);
-
-      vlog_node_t expr = vlog_value(conn);
-
-      ident_t id = vlog_ident(expr);
-      tree_t decl = NULL;
-
-      for (int j = 0; j < outports; j++) {
-         tree_t p = tree_port(ctx->out, j);
-         if (tree_ident(p) == id) {
-            decl = p;
-            break;
-         }
-      }
-
-      if (decl == NULL) {
-         for (int j = 0; j < outdecls; j++) {
-            tree_t d = tree_decl(ctx->out, j);
-            if (tree_ident(d) == id) {
-               decl = d;
-               break;
-            }
-         }
-      }
-
-      assert(decl != NULL);
-
-      tree_t port = tree_port(ei->block, i);
-
-      type_t dtype = tree_type(decl);
-      type_t ptype = tree_type(port);
-
-      if (type_eq(dtype, ptype)) {
-         if (have_named)
-            add_param(bind, make_ref(decl), P_NAMED, make_ref(port));
-         else
-            add_param(bind, make_ref(decl), P_POS, NULL);
-      }
-      else if (tree_subkind(port) == PORT_IN) {
-         tree_t func = elab_verilog_conversion(dtype, ptype);
-         assert(func != NULL);
-
-         tree_t conv = tree_new(T_CONV_FUNC);
-         tree_set_loc(conv, vlog_loc(conn));
-         tree_set_ref(conv, func);
-         tree_set_ident(conv, tree_ident(func));
-         tree_set_type(conv, type_result(tree_type(func)));
-         tree_set_value(conv, make_ref(decl));
-
-         if (have_named)
-            add_param(bind, conv, P_NAMED, make_ref(port));
-         else
-            add_param(bind, conv, P_POS, NULL);
-      }
-      else {
-         tree_t func = elab_verilog_conversion(ptype, dtype);
-         assert(func != NULL);
-
-         tree_t conv = tree_new(T_CONV_FUNC);
-         tree_set_loc(conv, vlog_loc(conn));
-         tree_set_ref(conv, func);
-         tree_set_ident(conv, tree_ident(func));
-         tree_set_type(conv, type_result(tree_type(func)));
-         tree_set_value(conv, make_ref(port));
-
-         add_param(bind, make_ref(decl), P_NAMED, conv);
-         have_named = true;
-      }
-   }
-
    return bind;
 }
 
@@ -702,11 +550,8 @@ static tree_t elab_default_binding(tree_t inst, const elab_ctx_t *ctx)
 
    object_t *obj = lib_get_generic(lib, full_i, NULL);
 
-   vlog_node_t mod = vlog_from_object(obj);
-   if (mod != NULL) {
-      mod_cache_t *mc = elab_cached_module(mod, ctx);
-      return elab_mixed_binding(comp, mc);
-   }
+   if (vlog_from_object(obj) != NULL)
+      return NULL;  // Verilog binding handled separately
 
    tree_t entity = tree_from_object(obj);
 
@@ -1386,20 +1231,26 @@ static void elab_context(tree_t t)
 
 static void elab_inherit_context(elab_ctx_t *ctx, const elab_ctx_t *parent)
 {
-   ctx->parent    = parent;
-   ctx->jit       = parent->jit;
-   ctx->registry  = parent->registry;
-   ctx->mir       = parent->mir;
-   ctx->root      = parent->root;
-   ctx->dotted    = ctx->dotted ?: parent->dotted;
-   ctx->library   = ctx->library ?: parent->library;
-   ctx->out       = ctx->out ?: parent->out;
-   ctx->cover     = parent->cover;
-   ctx->sdf       = parent->sdf;
-   ctx->inst      = ctx->inst ?: parent->inst;
-   ctx->modcache  = parent->modcache;
-   ctx->depth     = parent->depth + 1;
-   ctx->model     = parent->model;
+   ctx->parent   = parent;
+   ctx->jit      = parent->jit;
+   ctx->registry = parent->registry;
+   ctx->mir      = parent->mir;
+   ctx->root     = parent->root;
+   ctx->dotted   = ctx->dotted ?: parent->dotted;
+   ctx->library  = ctx->library ?: parent->library;
+   ctx->out      = ctx->out ?: parent->out;
+   ctx->cover    = parent->cover;
+   ctx->sdf      = parent->sdf;
+   ctx->inst     = ctx->inst ?: parent->inst;
+   ctx->modcache = parent->modcache;
+   ctx->depth    = parent->depth + 1;
+   ctx->model    = parent->model;
+   ctx->errors   = error_count();
+}
+
+static bool elab_new_errors(const elab_ctx_t *ctx)
+{
+   return error_count() - ctx->errors;
 }
 
 static driver_set_t *elab_driver_set(const elab_ctx_t *ctx)
@@ -1412,9 +1263,12 @@ static driver_set_t *elab_driver_set(const elab_ctx_t *ctx)
       return NULL;
 }
 
-static void elab_lower(tree_t b, const elab_instance_t *ei, elab_ctx_t *ctx)
+static void elab_lower(tree_t b, elab_ctx_t *ctx)
 {
-   if (ei != NULL)
+   tree_t hier = tree_decl(b, 0);
+   assert(tree_kind(hier) == T_HIER);
+
+   if (tree_subkind(hier) == T_VERILOG)
       vlog_lower_block(ctx->mir, ctx->parent->dotted, b);
    else
       ctx->lowered = lower_instance(ctx->registry, ctx->parent->lowered,
@@ -1452,20 +1306,111 @@ static void elab_verilog_module(tree_t bind, ident_t label,
    if (bind != NULL)
       elab_ports(ei->block, bind, &new_ctx);
 
-   if (error_count() == 0)
+   if (elab_new_errors(&new_ctx) == 0)
       elab_decls(ei->block, &new_ctx);
 
-   if (error_count() == 0) {
+   if (elab_new_errors(&new_ctx) == 0) {
       new_ctx.drivers = find_drivers(ei->block);
-      elab_lower(b, ei, &new_ctx);
+      elab_lower(b, &new_ctx);
    }
 
-   if (error_count() == 0)
+   if (elab_new_errors(&new_ctx) == 0)
       elab_verilog_stmts(ei->body, &new_ctx);
 
    elab_pop_scope(&new_ctx);
 }
 
+static void elab_verilog_ports(vlog_node_t inst, elab_instance_t *ei,
+                               const elab_ctx_t *ctx)
+{
+   assert(vlog_kind(inst) == V_MOD_INST);
+
+   const int nports = vlog_ports(ei->body);
+   const int nparams = vlog_params(inst);
+
+   if (nports != nparams) {
+      error_at(vlog_loc(inst), "expected %d port connections for module %s "
+               "but found %d", nports, istr(vlog_ident2(ei->body)), nparams);
+      return;
+   }
+
+   ident_t block_name = vlog_ident(inst);
+
+   for (int i = 0; i < nports; i++) {
+      vlog_node_t port = vlog_ref(vlog_port(ei->body, i));
+      ident_t port_name = vlog_ident(port);
+
+      vlog_node_t conn = vlog_param(inst, i);
+      assert(vlog_kind(conn) == V_PORT_CONN);
+
+      if (vlog_has_ident(conn) && vlog_ident(conn) != port_name) {
+         bool found = false;
+         for (int j = 0; j < nparams && !found; j++) {
+            if (vlog_ident((conn = vlog_param(inst, j))) == port_name)
+               found = true;
+         }
+
+         if (!found) {
+            diag_t *d = diag_new(DIAG_ERROR, vlog_loc(inst));
+            diag_printf(d, "missing port connection for '%s'", istr(port_name));
+            diag_hint(d, vlog_loc(port), "'%s' declared here", istr(port_name));
+            diag_emit(d);
+            continue;
+         }
+      }
+
+      const loc_t *loc = vlog_loc(conn);
+      ident_t name = ident_uniq("#assign#%s.%s", istr(block_name),
+                                istr(port_name));
+
+      vlog_node_t ref = vlog_new(V_HIER_REF);
+      vlog_set_ident(ref, port_name);
+      vlog_set_ref(ref, port);
+      vlog_set_ident2(ref, block_name);
+
+      vlog_node_t assign = vlog_new(V_ASSIGN), target;
+      vlog_set_ident(assign, name);
+
+      switch (vlog_subkind(port)) {
+      case V_PORT_INPUT:
+         vlog_set_target(assign, (target = ref));
+         vlog_set_value(assign, vlog_value(conn));
+         break;
+      case V_PORT_OUTPUT:
+         vlog_set_target(assign, (target = vlog_value(conn)));
+         vlog_set_value(assign, ref);
+         break;
+      default:
+         fatal_at(vlog_loc(port), "sorry, this port kind is not supported");
+      }
+
+      if (!vlog_is_net(target)) {
+         diag_t *d = diag_new(DIAG_ERROR, vlog_loc(conn));
+         if (vlog_kind(target) == V_REF)
+            diag_printf(d, "'%s'", istr(vlog_ident(target)));
+         else
+            diag_printf(d, "expression");
+         diag_printf(d, " cannot be driven by continuous assignment from "
+                     "port '%s'", istr(vlog_ident(port)));
+         diag_emit(d);
+      }
+
+      vlog_set_loc(assign, loc);
+
+      tree_t wrap = tree_new(T_VERILOG);
+      tree_set_ident(wrap, name);
+      tree_set_vlog(wrap, assign);
+      tree_set_loc(wrap, loc);
+
+      tree_add_stmt(ctx->out, wrap);
+
+      ident_t sym = ident_prefix(ctx->dotted, name, '.');
+
+      mir_defer(ctx->mir, sym, ctx->dotted, MIR_UNIT_PROCESS,
+                vlog_lower_deferred, vlog_to_object(assign));
+   }
+}
+
 static void elab_verilog_instance_list(vlog_node_t v, const elab_ctx_t *ctx)
 {
    ident_t modname = vlog_ident(v);
@@ -1510,12 +1455,87 @@ static void elab_verilog_instance_list(vlog_node_t v, const elab_ctx_t *ctx)
       vlog_node_t inst = vlog_stmt(v, i);
       assert(vlog_kind(inst) == V_MOD_INST);
 
-      tree_t bind = elab_verilog_binding(inst, ei, ctx);
-      if (bind != NULL)
-         elab_verilog_module(bind, vlog_ident(inst), ei, ctx);
+      elab_verilog_module(NULL, vlog_ident(inst), ei, ctx);
+      elab_verilog_ports(inst, ei, ctx);
    }
 }
 
+static void elab_verilog_block(vlog_node_t v, const elab_ctx_t *ctx)
+{
+   ident_t id = vlog_ident(v);
+
+   tree_t b = tree_new(T_BLOCK);
+   tree_set_ident(b, id);
+   tree_set_loc(b, vlog_loc(v));
+
+   tree_add_stmt(ctx->out, b);
+
+   ident_t ndotted = ident_prefix(ctx->dotted, id, '.');
+
+   elab_ctx_t new_ctx = {
+      .out    = b,
+      .dotted = ndotted,
+   };
+   elab_inherit_context(&new_ctx, ctx);
+
+   tree_t wrap = tree_new(T_VERILOG);
+   tree_set_ident(wrap, id);
+   tree_set_loc(wrap, vlog_loc(v));
+   tree_set_vlog(wrap, v);
+
+   elab_push_scope(wrap, &new_ctx);
+
+   vlog_trans(v, b);
+
+   elab_lower(b, &new_ctx);
+   elab_verilog_stmts(v, &new_ctx);
+
+   elab_pop_scope(&new_ctx);
+}
+
+static void *elab_verilog_for_generate_cb(jit_scalar_t *args, void *ctx)
+{
+   const uint64_t abits = args[0].integer;
+   const uint64_t bbits = args[1].integer;
+   return (void *)(uintptr_t)(abits == 1 && bbits == 0);
+}
+
+static void elab_verilog_for_generate(vlog_node_t v, const elab_ctx_t *ctx)
+{
+   mir_unit_t *init = vlog_lower_thunk(ctx->mir, ctx->dotted, vlog_left(v));
+   mir_unit_t *test = vlog_lower_thunk(ctx->mir, ctx->dotted, vlog_value(v));
+   mir_unit_t *step = vlog_lower_thunk(ctx->mir, ctx->dotted, vlog_right(v));
+
+   void *context = *mptr_get(ctx->scope->privdata);
+
+   jit_call_thunk2(ctx->jit, init, context, NULL, NULL);
+
+   vlog_node_t genvar = vlog_ref(vlog_target(vlog_stmt(vlog_left(v), 0)));
+   assert(vlog_kind(genvar) == V_GENVAR_DECL);
+
+   vlog_node_t s0 = vlog_stmt(v, 0);
+   assert(vlog_kind(s0) == V_BLOCK);
+
+   jit_handle_t handle = jit_compile(ctx->jit, ctx->dotted);
+   uint64_t *val = jit_get_frame_var(ctx->jit, handle, vlog_ident(genvar));
+
+   while (jit_call_thunk2(ctx->jit, test, context,
+                          elab_verilog_for_generate_cb, NULL)) {
+      const int32_t index = (int32_t)val[0];
+      // TODO: check not X/Z
+
+      vlog_node_t copy = vlog_generate_instance(s0, genvar, index, ctx->dotted);
+
+      elab_verilog_block(copy, ctx);
+
+      jit_call_thunk2(ctx->jit, step, context, NULL, NULL);
+   }
+
+   mir_unit_free(init);
+   mir_unit_free(test);
+   mir_unit_free(step);
+}
+
 static void elab_verilog_stmts(vlog_node_t v, const elab_ctx_t *ctx)
 {
    const int nstmts = vlog_stmts(v);
@@ -1552,13 +1572,19 @@ static void elab_verilog_stmts(vlog_node_t v, const elab_ctx_t *ctx)
          }
          break;
       case V_BLOCK:
-         // TODO: should create a block in hierarchy
-         elab_verilog_stmts(s, ctx);
+         elab_verilog_block(s, ctx);
          break;
       case V_IF_GENERATE:
          error_at(vlog_loc(s), "if-generate construct could not be "
                   "evaluated at elaboration time");
          break;
+      case V_FOR_GENERATE:
+         elab_verilog_for_generate(s, ctx);
+         break;
+      case V_SPECIFY:
+         warn_at(vlog_loc(s), "specify blocks are not currently supported and "
+                 "will be ignored");
+         break;
       default:
          fatal_at(vlog_loc(s), "sorry, this Verilog statement is not "
                   "currently supported");
@@ -1566,6 +1592,129 @@ static void elab_verilog_stmts(vlog_node_t v, const elab_ctx_t *ctx)
    }
 }
 
+static vlog_node_t elab_mixed_generics(tree_t comp, vlog_node_t mod,
+                                       const elab_ctx_t *ctx)
+{
+   vlog_node_t stmt = vlog_new(V_MOD_INST);
+   vlog_set_ident(stmt, vlog_ident2(mod));
+
+   vlog_node_t list = vlog_new(V_INST_LIST);
+   vlog_set_ident(list, vlog_ident2(mod));
+   vlog_add_stmt(list, stmt);
+
+   const int vhdl_ngenerics = tree_generics(comp);
+   const int vlog_ndecls = vlog_decls(mod);
+
+   LOCAL_BIT_MASK have;
+   mask_init(&have, vhdl_ngenerics);
+
+   bool named = false;
+   for (int i = 0; i < vlog_ndecls; i++) {
+      vlog_node_t d = vlog_decl(mod, i);
+      if (vlog_kind(d) != V_PARAM_DECL)
+         continue;
+
+      ident_t name = vlog_ident(d);
+
+      tree_t cg = NULL;
+      for (int j = 0; j < vhdl_ngenerics; j++) {
+         tree_t gj = tree_generic(comp, j);
+         if (ident_casecmp(tree_ident(gj), name)) {
+            cg = gj;
+            mask_set(&have, j);
+            break;
+         }
+      }
+
+      if (cg == NULL && vlog_has_value(d)) {
+         named = true;
+         continue;
+      }
+      else if (cg == NULL) {
+         error_at(tree_loc(comp), "missing matching VHDL generic declaration "
+                  "for Verilog parameter '%s' with no default value in "
+                  "component %s", istr(name), istr(tree_ident(comp)));
+         return NULL;
+      }
+
+      type_t ctype = tree_type(cg);
+      type_t etype = std_type(NULL, STD_INTEGER);
+
+      if (!type_eq(ctype, etype)) {
+         error_at(tree_loc(cg), "generic %s should have type %s to match "
+                  "corresponding Verilog parameter", istr(tree_ident(cg)),
+                  type_pp(etype));
+         return NULL;
+      }
+
+      tree_t value = hash_get(ctx->generics, cg);
+      if (value == NULL)
+         fatal_at(tree_loc(ctx->out), "cannot get value of generic %s",
+                  istr(tree_ident(cg)));
+
+      number_t n = number_from_int(assume_int(value));
+
+      vlog_node_t num = vlog_new(V_NUMBER);
+      vlog_set_number(num, n);
+
+      vlog_node_t pa = vlog_new(V_PARAM_ASSIGN);
+      vlog_set_value(pa, num);
+      if (named) vlog_set_ident(pa, vlog_ident(d));
+
+      vlog_add_param(list, pa);
+   }
+
+   for (int i = 0; i < vhdl_ngenerics; i++) {
+      if (!mask_test(&have, i)) {
+         tree_t g = tree_generic(comp, i);
+         diag_t *d = diag_new(DIAG_ERROR, tree_loc(g));
+         diag_printf(d, "generic %s not found in Verilog module %s",
+                     istr(tree_ident(g)), istr(vlog_ident2(mod)));
+         diag_emit(d);
+      }
+   }
+
+   return list;
+}
+
+static void elab_mixed_instance(tree_t inst, tree_t comp, vlog_node_t mod,
+                                const elab_ctx_t *ctx)
+{
+   ident_t ndotted = ident_prefix(ctx->dotted, tree_ident(inst), '.');
+
+   elab_ctx_t new_ctx = {
+      .dotted = ndotted,
+      .inst   = inst,
+   };
+   elab_inherit_context(&new_ctx, ctx);
+
+   tree_t b = tree_new(T_BLOCK);
+   tree_set_ident(b, tree_ident(inst));
+   tree_set_loc(b, tree_loc(inst));
+
+   tree_add_stmt(ctx->out, b);
+   new_ctx.out = b;
+
+   elab_push_scope(comp, &new_ctx);
+   elab_generics(comp, inst, &new_ctx);
+   elab_instance_fixup(comp, &new_ctx);
+   elab_ports(comp, inst, &new_ctx);
+
+   if (elab_new_errors(&new_ctx) == 0)
+      elab_lower(b, &new_ctx);
+
+   vlog_node_t list = elab_mixed_generics(comp, mod, &new_ctx);
+   if (list != NULL) {
+      elab_instance_t *ei = elab_new_instance(mod, list, &new_ctx);
+
+      tree_t bind = elab_mixed_binding(comp, ei);
+      if (bind != NULL && elab_new_errors(&new_ctx) == 0)
+         elab_verilog_module(bind, vlog_ident2(mod), ei, &new_ctx);
+   }
+
+   elab_pop_scope(&new_ctx);
+}
+
 static void elab_architecture(tree_t bind, tree_t arch, tree_t config,
                               const elab_ctx_t *ctx)
 {
@@ -1629,7 +1778,7 @@ static void elab_architecture(tree_t bind, tree_t arch, tree_t config,
 
    if (error_count() == 0) {
       new_ctx.drivers = find_drivers(arch_copy);
-      elab_lower(b, NULL, &new_ctx);
+      elab_lower(b, &new_ctx);
       elab_stmts(entity, &new_ctx);
       elab_stmts(arch_copy, &new_ctx);
    }
@@ -1705,8 +1854,22 @@ static void elab_component(tree_t inst, tree_t comp, const elab_ctx_t *ctx)
          arch = tree_ref(config);
       }
    }
-   else if (spec == NULL && (bind = elab_default_binding(inst, ctx)))
+   else if (spec != NULL)
+      ;   // Binding indication is open
+   else if ((bind = elab_default_binding(inst, ctx)))
       arch = tree_ref(bind);
+   else {
+      ident_t prefix = lib_name(ctx->library);
+      ident_t qual = ident_prefix(prefix, tree_ident(comp), '.');
+
+      object_t *obj = lib_get_generic(ctx->library, qual, NULL);
+
+      vlog_node_t mod = vlog_from_object(obj);
+      if (mod != NULL) {
+         elab_mixed_instance(inst, comp, mod, ctx);
+         return;
+      }
+   }
 
    // Must create a unique instance if type or package generics present
    const int ngenerics = tree_generics(comp);
@@ -1744,57 +1907,13 @@ static void elab_component(tree_t inst, tree_t comp, const elab_ctx_t *ctx)
    elab_instance_fixup(comp, &new_ctx);
    elab_ports(comp, inst, &new_ctx);
 
-   if (bind != NULL && tree_kind(arch) != T_VERILOG)
+   if (bind != NULL)
       new_ctx.drivers = find_drivers(bind);
 
    if (error_count() == 0)
-      elab_lower(b, NULL, &new_ctx);
+      elab_lower(b, &new_ctx);
 
-   if (error_count() > 0 || arch == NULL)
-      ;   // Unbound architecture
-   else if (tree_kind(arch) == T_VERILOG) {
-      vlog_node_t mod = tree_vlog(arch);
-
-      vlog_node_t stmt = vlog_new(V_MOD_INST);
-      vlog_set_ident(stmt, vlog_ident2(mod));
-
-      vlog_node_t list = vlog_new(V_INST_LIST);
-      vlog_set_ident(list, vlog_ident2(mod));
-      vlog_add_stmt(list, stmt);
-
-      const int ngenmaps = tree_genmaps(bind);
-      for (int i = 0; i < ngenmaps; i++) {
-         tree_t decl = tree_ref(tree_value(tree_genmap(bind, i)));
-         assert(tree_kind(decl) == T_GENERIC_DECL);
-
-         tree_t map = hash_get(new_ctx.generics, decl);
-         if (map == NULL)
-            fatal_at(tree_loc(inst), "cannot get value of generic %s",
-                     istr(tree_ident(decl)));
-
-         const int64_t ival = assume_int(map);
-
-         // TODO: have some better way of constructing these
-         char buf[64];
-         checked_sprintf(buf, sizeof(buf), "%lld", llabs(ival));
-
-         number_t n = number_new(buf, NULL);
-         if (ival < 0)
-            n = number_negate(n);
-
-         vlog_node_t num = vlog_new(V_NUMBER);
-         vlog_set_number(num, n);
-
-         vlog_node_t pa = vlog_new(V_PARAM_ASSIGN);
-         vlog_set_value(pa, num);
-
-         vlog_add_param(list, pa);
-      }
-
-      elab_instance_t *ei = elab_new_instance(mod, list, &new_ctx);
-      elab_verilog_module(bind, vlog_ident2(mod), ei, &new_ctx);
-   }
-   else
+   if (arch != NULL && error_count() == 0)
       elab_architecture(bind, arch, config, &new_ctx);
 
    elab_pop_scope(&new_ctx);
@@ -1933,12 +2052,6 @@ static inline tree_t elab_eval_expr(tree_t t, const elab_ctx_t *ctx)
    return eval_must_fold(ctx->jit, t, ctx->registry, ctx->lowered, context);
 }
 
-static bool elab_copy_genvar_cb(tree_t t, void *ctx)
-{
-   tree_t genvar = ctx;
-   return tree_kind(t) == T_REF && tree_ref(t) == genvar;
-}
-
 static void elab_generate_range(tree_t r, int64_t *low, int64_t *high,
                                 const elab_ctx_t *ctx)
 {
@@ -1988,17 +2101,15 @@ static void elab_for_generate(tree_t t, const elab_ctx_t *ctx)
    tree_t g = tree_decl(t, 0);
    assert(tree_kind(g) == T_GENERIC_DECL);
 
-   ident_t base = tree_ident(t);
+   ident_t prefix = ident_prefix(ctx->dotted, tree_ident(t), '.');
 
    for (int64_t i = low; i <= high; i++) {
-      LOCAL_TEXT_BUF tb = tb_new();
-      tb_cat(tb, istr(base));
-      tb_printf(tb, "(%"PRIi64")", i);
+      ident_t id = ident_sprintf("%s(%"PRIi64")", istr(tree_ident(t)), i);
+      ident_t ndotted = ident_prefix(ctx->dotted, id, '.');
 
-      ident_t id = ident_new(tb_get(tb));
+      tree_t copy = vhdl_generate_instance(t, prefix, ndotted);
 
       tree_t b = tree_new(T_BLOCK);
-      tree_set_loc(b, tree_loc(t));
       tree_set_ident(b, id);
       tree_set_loc(b, tree_loc(t));
 
@@ -2012,8 +2123,6 @@ static void elab_for_generate(tree_t t, const elab_ctx_t *ctx)
       tree_add_generic(b, g);
       tree_add_genmap(b, map);
 
-      ident_t ndotted = ident_prefix(ctx->dotted, id, '.');
-
       elab_ctx_t new_ctx = {
          .out       = b,
          .dotted    = ndotted,
@@ -2022,14 +2131,6 @@ static void elab_for_generate(tree_t t, const elab_ctx_t *ctx)
       };
       elab_inherit_context(&new_ctx, ctx);
 
-      new_ctx.prefix[0] = ident_prefix(ctx->dotted, base, '.');
-
-      tree_t roots[] = { t };
-      copy_with_renaming(roots, 1, elab_copy_genvar_cb, NULL, g, ndotted,
-                         new_ctx.prefix, ARRAY_LEN(new_ctx.prefix));
-
-      tree_t copy = roots[0];
-
       elab_push_scope(t, &new_ctx);
       hash_put(new_ctx.generics, g, tree_value(map));
 
@@ -2042,7 +2143,7 @@ static void elab_for_generate(tree_t t, const elab_ctx_t *ctx)
          elab_decls(copy, &new_ctx);
 
       if (error_count() == 0) {
-         elab_lower(b, NULL, &new_ctx);
+         elab_lower(b, &new_ctx);
          elab_stmts(copy, &new_ctx);
       }
 
@@ -2093,7 +2194,7 @@ static void elab_if_generate(tree_t t, const elab_ctx_t *ctx)
          new_ctx.drivers = find_drivers(cond);
 
          if (error_count() == 0) {
-            elab_lower(b, NULL, &new_ctx);
+            elab_lower(b, &new_ctx);
             elab_stmts(cond, &new_ctx);
          }
 
@@ -2132,7 +2233,7 @@ static void elab_case_generate(tree_t t, const elab_ctx_t *ctx)
    new_ctx.drivers = find_drivers(chosen);
 
    if (error_count() == 0) {
-      elab_lower(b, NULL, &new_ctx);
+      elab_lower(b, &new_ctx);
       elab_stmts(chosen, &new_ctx);
    }
 
@@ -2208,15 +2309,13 @@ static void elab_block(tree_t t, const elab_ctx_t *ctx)
    };
    elab_inherit_context(&new_ctx, ctx);
 
-   const int base_errors = error_count();
-
    elab_push_scope(t, &new_ctx);
    elab_generics(t, t, &new_ctx);
    elab_ports(t, t, &new_ctx);
    elab_decls(t, &new_ctx);
 
-   if (error_count() == base_errors) {
-      elab_lower(b, NULL, &new_ctx);
+   if (elab_new_errors(&new_ctx) == 0) {
+      elab_lower(b, &new_ctx);
       elab_stmts(t, &new_ctx);
    }
 
