@@ -178,7 +178,7 @@ static inline void *interp_get_pointer(jit_interp_t *state, jit_value_t value)
    case JIT_ADDR_ABS:
       return (void *)(intptr_t)value.int64;
    case JIT_ADDR_COVER:
-      return jit_get_cover_ptr(state->func->jit, value);
+      return jit_get_cover_ptr(state->func, value);
    default:
       CANNOT_HANDLE(value);
    }
@@ -214,7 +214,7 @@ static inline jit_scalar_t interp_get_scalar(jit_interp_t *state,
       return (jit_scalar_t){ .pointer = (void *)(intptr_t)value.int64 };
    case JIT_ADDR_COVER:
       return (jit_scalar_t){
-         .pointer = jit_get_cover_ptr(state->func->jit, value)
+         .pointer = jit_get_cover_ptr(state->func, value)
       };
    default:
       CANNOT_HANDLE(value);
@@ -412,10 +412,18 @@ static void interp_fadd(jit_interp_t *state, jit_ir_t *ir)
 
 static void interp_shl(jit_interp_t *state, jit_ir_t *ir)
 {
-   const int64_t arg1 = interp_get_int(state, ir->arg1);
-   const int64_t arg2 = interp_get_int(state, ir->arg2);
+   const uint64_t arg1 = interp_get_int(state, ir->arg1);
+   const uint64_t arg2 = interp_get_int(state, ir->arg2);
 
-   state->regs[ir->result].integer = arg1 << arg2;
+   state->regs[ir->result].integer = arg2 < 64 ? arg1 << arg2 : 0;
+}
+
+static void interp_shr(jit_interp_t *state, jit_ir_t *ir)
+{
+   const uint64_t arg1 = interp_get_int(state, ir->arg1);
+   const uint64_t arg2 = interp_get_int(state, ir->arg2);
+
+   state->regs[ir->result].integer = arg2 < 64 ? arg1 >> arg2 : 0;
 }
 
 static void interp_asr(jit_interp_t *state, jit_ir_t *ir)
@@ -423,7 +431,10 @@ static void interp_asr(jit_interp_t *state, jit_ir_t *ir)
    const int64_t arg1 = interp_get_int(state, ir->arg1);
    const int64_t arg2 = interp_get_int(state, ir->arg2);
 
-   state->regs[ir->result].integer = arg1 >> arg2;
+   if (arg2 < 64)
+      state->regs[ir->result].integer = arg1 >> arg2;
+   else
+      state->regs[ir->result].integer = arg1 < 0 ? -1 : 0;
 }
 
 static void interp_store(jit_interp_t *state, jit_ir_t *ir)
@@ -886,31 +897,21 @@ static void interp_sadd(jit_interp_t *state, jit_ir_t *ir)
 static void interp_pack(jit_interp_t *state, jit_ir_t *ir)
 {
    const uint8_t *src = interp_get_pointer(state, ir->arg1);
-   const size_t size = interp_get_int(state, ir->arg2);
-   assert(size <= 64);  // TODO
-
-   uint64_t abits = 0, bbits = 0;
-
-   for (size_t i = 0; i < size; i++) {
-      abits = (abits << 1) | (src[i] & 1);
-      bbits = (bbits << 1) | ((src[i] >> 1) & 1);
-   }
-
-   state->args[0].integer = abits;
-   state->args[1].integer = bbits;
+   const int size = interp_get_int(state, ir->arg2);
+   __nvc_pack(src, size, state->args);
 }
 
 static void interp_unpack(jit_interp_t *state, jit_ir_t *ir)
 {
-   uint8_t *dest = state->args[0].pointer;
-   size_t size = state->args[1].integer;
-   uint8_t strength = state->args[2].integer;
+   jit_scalar_t aval = interp_get_scalar(state, ir->arg1);
+   jit_scalar_t bval = interp_get_scalar(state, ir->arg2);
+   __nvc_unpack(aval, bval, state->args);
+}
 
-   uint64_t abits = interp_get_int(state, ir->arg1);
-   uint64_t bbits = interp_get_int(state, ir->arg2);
-
-   for (size_t i = 0; i < size; i++, abits >>= 1, bbits >>= 1)
-      dest[size - i - 1] = (abits & 1) | ((bbits & 1) << 1) | strength;
+static void interp_vec4op(jit_interp_t *state, jit_ir_t *ir)
+{
+   state->anchor->irpos = ir - state->func->irbuf;
+   __nvc_vec4op(ir->arg1.int64, state->anchor, state->args, ir->arg2.int64);
 }
 
 static void interp_loop(jit_interp_t *state)
@@ -961,6 +962,9 @@ static void interp_loop(jit_interp_t *state)
       case J_SHL:
          interp_shl(state, ir);
          break;
+      case J_SHR:
+         interp_shr(state, ir);
+         break;
       case J_ASR:
          interp_asr(state, ir);
          break;
@@ -1086,6 +1090,9 @@ static void interp_loop(jit_interp_t *state)
       case MACRO_UNPACK:
          interp_unpack(state, ir);
          break;
+      case MACRO_VEC4OP:
+         interp_vec4op(state, ir);
+         break;
       default:
          interp_dump(state);
          fatal_trace("cannot interpret opcode %s", jit_op_name(ir->op));
