@@ -20,6 +20,7 @@
 #include "jit/jit-priv.h"
 #include "jit/jit.h"
 #include "option.h"
+#include "rt/assert.h"
 #include "thread.h"
 
 #include <assert.h>
@@ -124,6 +125,14 @@ static const uint8_t or_table[16][16] = {
    {    _U, _X, _X, _1, _X, _X, _X, _1, _X   },  // | - |
 };
 
+#if defined HAVE_SSE41
+static const uint8_t not_table[1][16] = {
+   // ---------------------------------------------------
+   // |  U   X   0   1   Z   W   L   H   -          |   |
+   {    _U, _X, _1, _0, _X, _X, _1, _0, _X   },
+};
+#endif
+
 #if defined HAVE_SSE41 || defined HAVE_NEON
 
 // Compressed lookup tables for vectorised intrinsics.  Note the
@@ -244,12 +253,9 @@ static void print_m128i(const char *tag, __m128i vec)
 #endif
 
 __attribute__((cold, noinline))
-static void ieee_warn(jit_func_t *func, jit_anchor_t *caller,
-                      const char *fmt, ...)
+static void ieee_msg_v(jit_func_t *func, jit_anchor_t *caller,
+                       vhdl_severity_t severity, const char *fmt, va_list ap)
 {
-   if (!opt_get_int(OPT_IEEE_WARNINGS))
-      return;
-
    jit_anchor_t frame = {
       .caller = caller,
       .func = func
@@ -258,37 +264,40 @@ static void ieee_warn(jit_func_t *func, jit_anchor_t *caller,
    jit_thread_local_t *thread = jit_thread_local();
    thread->anchor = &frame;
 
-   va_list ap;
-   va_start(ap, fmt);
-
-   diag_t *d = diag_new(DIAG_WARN, NULL);
+   diag_t *d = diag_new(get_diag_severity(severity), NULL);
    diag_vprintf(d, fmt, ap);
    diag_show_source(d, false);
-   diag_emit(d);
 
-   va_end(ap);
+   emit_vhdl_diag(d, severity);
 
    thread->anchor = NULL;
 }
 
-__attribute__((cold, noreturn))
-static void __ieee_failure(jit_func_t *func, jit_anchor_t *caller,
-                           const char *msg)
+__attribute__((cold, noinline))
+static void __ieee_msg(jit_func_t *func, jit_anchor_t *caller,
+                       vhdl_severity_t severity, const char *fmt, ...)
 {
-   jit_anchor_t frame = {
-      .caller = caller,
-      .func = func
-   };
+   va_list ap;
+   va_start(ap, fmt);
 
-   jit_thread_local_t *thread = jit_thread_local();
-   thread->anchor = &frame;
+   ieee_msg_v(func, caller, severity, fmt, ap);
 
-   diag_t *d = diag_new(DIAG_FATAL, NULL);
-   diag_printf(d, "Assertion Failure: %s", msg);
-   diag_show_source(d, false);
-   diag_emit(d);
+   va_end(ap);
+}
 
-   jit_abort_with_status(EXIT_FAILURE);
+__attribute__((cold, noinline))
+static void ieee_warn(jit_func_t *func, jit_anchor_t *caller,
+                      const char *fmt, ...)
+{
+   if (opt_get_int(OPT_IEEE_WARNINGS) != IEEE_WARNINGS_ON)
+      return;
+
+   va_list ap;
+   va_start(ap, fmt);
+
+   ieee_msg_v(func, caller, SEVERITY_WARNING, fmt, ap);
+
+   va_end(ap);
 }
 
 __attribute__((always_inline))
@@ -553,8 +562,8 @@ static inline uint8_t *__to_unsigned(jit_func_t *func, jit_anchor_t *anchor,
 static void ieee_plus_unsigned(jit_func_t *func, jit_anchor_t *anchor,
                                jit_scalar_t *args, tlab_t *tlab)
 {
-   const int lsize = args[3].integer ^ (args[3].integer >> 63);
-   const int rsize = args[6].integer ^ (args[6].integer >> 63);
+   const int lsize = ffi_array_length(args[3].integer);
+   const int rsize = ffi_array_length(args[6].integer);
    uint8_t *left = args[1].pointer;
    uint8_t *right = args[4].pointer;
 
@@ -847,6 +856,73 @@ static void ieee_mul_signed(jit_func_t *func, jit_anchor_t *anchor,
    }
 }
 
+static void ieee_divmod(jit_func_t *func, jit_anchor_t *anchor,
+                        jit_scalar_t *args, tlab_t *tlab)
+{
+   uint8_t *num = args[2].pointer;
+   const int num_size = ffi_array_length(args[4].integer);
+   uint8_t *denom = args[5].pointer;
+   const int denom_size = ffi_array_length(args[7].integer);
+   uint8_t *xquot = args[8].pointer;
+   uint8_t *xremain = args[11].pointer;
+
+   assert(ffi_array_length(args[10].integer) == num_size);
+   assert(ffi_array_length(args[13].integer) == denom_size);
+
+   const uint32_t mark = __tlab_mark(tlab);
+
+   uint8_t *temp = __tlab_alloc(tlab, num_size + 1, 16);
+   temp[0] = _0;
+   memcpy(temp + 1, num, num_size);
+
+   const int quot_size = MAX(num_size, denom_size);
+   uint8_t *quot = __tlab_alloc(tlab, quot_size, 16);
+   memset(quot, _0, quot_size);
+
+   int topbit = -1;
+   for (int j = denom_size - 1; j >= 0; j--) {
+      if (denom[denom_size - 1 - j] == _1) {
+         topbit = j;
+         break;
+      }
+   }
+
+   if (unlikely(topbit < 0))
+      __ieee_msg(func, anchor, SEVERITY_ERROR,
+                 "NUMERIC_STD.DIVMOD: DIV, MOD, or REM by zero");
+   else {
+      uint8_t *denom2 = __tlab_alloc(tlab, topbit + 2, 16);
+      denom2[0] = _0;
+      memcpy(denom2 + 1, denom + denom_size - 1 - topbit , topbit + 1);
+
+      uint8_t *denom3 = __tlab_alloc(tlab, topbit + 2, 16);
+      __invert_bits(denom2, topbit + 2, denom3);
+
+      for (int j = num_size - (topbit + 1); j >= 0; j--) {
+         uint8_t *slice = temp + num_size - (topbit + j + 1);
+
+         int pos = 0;
+         for (; pos < topbit + 1 && slice[pos] == denom2[pos]; pos++);
+
+         if (slice[pos] >= denom2[pos]) {
+            __ieee_packed_add(slice, denom3, topbit + 2, 1, slice);
+            quot[quot_size - 1 - j] = _1;
+         }
+      }
+   }
+
+   memcpy(xquot, quot + quot_size - num_size, num_size);
+
+   if (denom_size > num_size + 1) {
+      memset(xremain, _0, denom_size - (num_size + 1));
+      memcpy(xremain + denom_size - (num_size + 1), temp, num_size + 1);
+   }
+   else
+      memcpy(xremain, temp + num_size + 1 - denom_size, denom_size);
+
+   __tlab_restore(tlab, mark);
+}
+
 static bool ieee_unsigned_cmp(jit_func_t *func, jit_anchor_t *anchor,
                               jit_scalar_t *args, tlab_t *tlab,
                               uint8_t *lbyte, uint8_t *rbyte, const char *op)
@@ -941,8 +1017,9 @@ static void ieee_and_vector_sse41(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"and\": arguments of "
-                     "overloaded 'and' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"and\": arguments of overloaded 'and' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -977,8 +1054,9 @@ static void ieee_and_vector_neon(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"and\": arguments of "
-                     "overloaded 'and' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"and\": arguments of overloaded 'and' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -1012,8 +1090,9 @@ static void ieee_and_vector(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"and\": arguments of "
-                     "overloaded 'and' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"and\": arguments of overloaded 'and' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, lsize, 8);
 
@@ -1037,8 +1116,9 @@ static void ieee_or_vector_sse41(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"or\": arguments of "
-                     "overloaded 'or' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"or\": arguments of overloaded 'or' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -1073,8 +1153,9 @@ static void ieee_or_vector_neon(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"or\": arguments of "
-                     "overloaded 'or' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"or\": arguments of overloaded 'or' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -1108,8 +1189,9 @@ static void ieee_or_vector(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"or\": arguments of "
-                     "overloaded 'or' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"or\": arguments of overloaded 'or' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -1133,8 +1215,9 @@ static void ieee_xor_vector_sse41(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"xor\": arguments of "
-                     "overloaded 'xor' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"xor\": arguments of overloaded 'xor' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -1169,8 +1252,9 @@ static void ieee_xor_vector_neon(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"xor\": arguments of "
-                     "overloaded 'xor' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"xor\": arguments of overloaded 'xor' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
 
@@ -1204,8 +1288,9 @@ static void std_xor_vector(jit_func_t *func, jit_anchor_t *anchor,
    uint8_t *right = args[4].pointer;
 
    if (unlikely(lsize != rsize))
-      __ieee_failure(func, anchor, "STD_LOGIC_1164.\"xor\": arguments of "
-                     "overloaded 'xor' operator are not of the same length");
+      __ieee_msg(func, anchor, SEVERITY_FAILURE,
+                 "STD_LOGIC_1164.\"xor\": arguments of overloaded 'xor' "
+                 "operator are not of the same length");
    else {
       uint8_t *result = __tlab_alloc(tlab, lsize, 8);
 
@@ -1218,6 +1303,30 @@ static void std_xor_vector(jit_func_t *func, jit_anchor_t *anchor,
    }
 }
 
+#ifdef HAVE_SSE41
+__attribute__((target("sse4.1")))
+static void ieee_not_vector_sse41(jit_func_t *func, jit_anchor_t *anchor,
+                                  jit_scalar_t *args, tlab_t *tlab)
+{
+   const int lsize = ffi_array_length(args[3].integer);
+   uint8_t *left = args[1].pointer;
+
+   uint8_t *result = __tlab_alloc(tlab, ALIGN_UP(lsize, 16), 16);
+
+   __m128i not_tbl = _mm_load_si128((const __m128i *)not_table);
+
+   for (int pos = 0; pos < lsize; pos += 16) {
+      __m128i input  = _mm_loadu_si128((const __m128i *)(left + pos));
+      __m128i output = _mm_shuffle_epi8(not_tbl, input);
+      _mm_store_si128((__m128i *)(result + pos), output);
+   }
+
+   args[0].pointer = result;
+   args[1].integer = 1;
+   args[2].integer = lsize;
+}
+#endif
+
 static void ieee_to_unsigned(jit_func_t *func, jit_anchor_t *anchor,
                              jit_scalar_t *args, tlab_t *tlab)
 {
@@ -1826,6 +1935,8 @@ static jit_intrinsic_t intrinsic_list[] = {
    { NS "\"*\"(" UU UU ")" UU, ieee_mul_unsigned },
    { NS "\"*\"(" S S ")" S, ieee_mul_signed },
    { NS "\"*\"(" US US ")" US, ieee_mul_signed },
+   { NS "DIVMOD(" UU UU UU UU ")", ieee_divmod },
+   { NS "DIVMOD(" U U U U ")", ieee_divmod },
    { NS "\"<\"(" U U ")B", ieee_less_unsigned },
    { NS "\"<\"(" UU UU ")B" , ieee_less_unsigned },
    { NS "\">\"(" U U ")B", ieee_greater_unsigned },
@@ -1873,11 +1984,15 @@ static jit_intrinsic_t intrinsic_list[] = {
    { SL "\"xor\"(YY)Y", ieee_xor_vector_sse41, CPU_SSE41 },
 #endif
 #ifdef HAVE_NEON
-   { SL "\"neon\"(VV)V", ieee_xor_vector_neon, CPU_NEON },
-   { SL "\"neon\"(YY)Y", ieee_xor_vector_neon, CPU_NEON },
+   { SL "\"xor\"(VV)V", ieee_xor_vector_neon, CPU_NEON },
+   { SL "\"xor\"(YY)Y", ieee_xor_vector_neon, CPU_NEON },
 #endif
    { SL "\"xor\"(VV)V", std_xor_vector },
    { SL "\"xor\"(YY)Y", std_xor_vector },
+#ifdef HAVE_SSE41
+   { SL "\"not\"(V)V", ieee_not_vector_sse41, CPU_SSE41 },
+   { SL "\"not\"(Y)Y", ieee_not_vector_sse41, CPU_SSE41 },
+#endif
    { NS "TO_UNSIGNED(NN)" U, ieee_to_unsigned },
    { NS "TO_UNSIGNED(NN)" UU, ieee_to_unsigned },
    { NS "TO_SIGNED(IN)" S, ieee_to_signed },
