@@ -667,6 +667,19 @@ static void jit_lvn_shl(jit_ir_t *ir, lvn_state_t *state)
    int64_t lhs, rhs;
    if (lvn_can_fold(ir, state, &lhs, &rhs) && rhs >= 0 && rhs < 64)
       lvn_convert_mov(ir, state, LVN_CONST(lhs << rhs));
+   else if (lvn_is_const(ir->arg2, state, &rhs) && rhs == 0)
+      lvn_convert_mov(ir, state, ir->arg1);
+   else
+      jit_lvn_generic(ir, state, VN_INVALID);
+}
+
+static void jit_lvn_shr(jit_ir_t *ir, lvn_state_t *state)
+{
+   int64_t lhs, rhs;
+   if (lvn_can_fold(ir, state, &lhs, &rhs) && rhs >= 0 && rhs < 64)
+      lvn_convert_mov(ir, state, LVN_CONST(lhs >> rhs));
+   else if (lvn_is_const(ir->arg2, state, &rhs) && rhs == 0)
+      lvn_convert_mov(ir, state, ir->arg1);
    else
       jit_lvn_generic(ir, state, VN_INVALID);
 }
@@ -676,6 +689,8 @@ static void jit_lvn_asr(jit_ir_t *ir, lvn_state_t *state)
    int64_t lhs, rhs;
    if (lvn_can_fold(ir, state, &lhs, &rhs) && rhs >= 0 && rhs < 64)
       lvn_convert_mov(ir, state, LVN_CONST(lhs >> rhs));
+   else if (lvn_is_const(ir->arg2, state, &rhs) && rhs == 0)
+      lvn_convert_mov(ir, state, ir->arg1);
    else
       jit_lvn_generic(ir, state, VN_INVALID);
 }
@@ -977,6 +992,7 @@ void jit_do_lvn(jit_func_t *f)
       case J_OR:  jit_lvn_or(ir, &state); break;
       case J_XOR: jit_lvn_xor(ir, &state); break;
       case J_SHL: jit_lvn_shl(ir, &state); break;
+      case J_SHR: jit_lvn_shr(ir, &state); break;
       case J_ASR: jit_lvn_asr(ir, &state); break;
       case J_MOV: jit_lvn_mov(ir, &state); break;
       case J_CMP: jit_lvn_cmp(ir, &state); break;
@@ -993,6 +1009,10 @@ void jit_do_lvn(jit_func_t *f)
       case MACRO_MEMSET: jit_lvn_memset(ir, &state); break;
       case MACRO_EXP: jit_lvn_exp(ir, &state); break;
       case MACRO_EXIT:
+      case MACRO_VEC4OP:
+      case MACRO_VEC2OP:
+      case MACRO_PACK:
+      case MACRO_UNPACK:
       case J_CALL:
          lvn_kill_args(&state);
          // Fall-through
