@@ -1282,7 +1282,7 @@ static void jit_x86_get_copy(code_blob_t *blob, x86_operand_t dst,
       MOV(dst, PTR(blob->func->cpool + src.int64), __QWORD);
       break;
    case JIT_ADDR_COVER:
-      MOV(dst, PTR(jit_get_cover_ptr(blob->func->jit, src)), __QWORD);
+      MOV(dst, PTR(jit_get_cover_ptr(blob->func, src)), __QWORD);
       break;
    default:
       fatal_trace("cannot handle value kind %d in jit_x86_get", src.kind);
@@ -1323,7 +1323,7 @@ static x86_operand_t jit_x86_get(code_blob_t *blob, x86_operand_t tmp,
       MOV(tmp, PTR(blob->func->cpool + src.int64), __QWORD);
       return tmp;
    case JIT_ADDR_COVER:
-      MOV(tmp, PTR(jit_get_cover_ptr(blob->func->jit, src)), __QWORD);
+      MOV(tmp, PTR(jit_get_cover_ptr(blob->func, src)), __QWORD);
       return tmp;
    default:
       fatal_trace("cannot handle value kind %d in jit_x86_get", src.kind);
@@ -1345,7 +1345,7 @@ static x86_operand_t jit_x86_get_addr(code_blob_t *blob, jit_value_t addr,
       MOV(tmp, IMM(addr.int64), __QWORD);
       return ADDR(tmp, 0);
    case JIT_ADDR_COVER:
-      MOV(tmp, PTR(jit_get_cover_ptr(blob->func->jit, addr)), __QWORD);
+      MOV(tmp, PTR(jit_get_cover_ptr(blob->func, addr)), __QWORD);
       return ADDR(tmp, 0);
    default:
       fatal_trace("cannot handle value kind %d in jit_x86_get_addr", addr.kind);
