@@ -235,6 +235,7 @@ typedef enum {
    MIR_OP_FUNCTION_TRIGGER,
    MIR_OP_OR_TRIGGER,
    MIR_OP_CMP_TRIGGER,
+   MIR_OP_LEVEL_TRIGGER,
    MIR_OP_ADD_TRIGGER,
    MIR_OP_INSTANCE_NAME,
    MIR_OP_LAST_EVENT,
@@ -251,6 +252,13 @@ typedef enum {
    MIR_OP_BINARY,
    MIR_OP_UNARY,
    MIR_OP_DIR_CHECK,
+   MIR_OP_INSERT,
+   MIR_OP_TEST,
+   MIR_OP_EXTRACT,
+   MIR_OP_SCHED_PROCESS,
+   MIR_OP_SCHED_INACTIVE,
+   MIR_OP_SCHED_DEPOSIT,
+   MIR_OP_PUT_DRIVER,
 } mir_op_t;
 
 typedef enum {
@@ -326,11 +334,16 @@ typedef enum {
    MIR_VEC_LOG_NEQ,
    MIR_VEC_CASE_EQ,
    MIR_VEC_CASE_NEQ,
+   MIR_VEC_CASEX_EQ,
    MIR_VEC_ADD,
    MIR_VEC_SUB,
    MIR_VEC_MUL,
+   MIR_VEC_DIV,
+   MIR_VEC_MOD,
    MIR_VEC_SLL,
    MIR_VEC_SRL,
+   MIR_VEC_SLA,
+   MIR_VEC_SRA,
 } mir_vec_op_t;
 
 #define MIR_APPEND UINT_MAX
@@ -368,6 +381,7 @@ mir_type_t mir_get_elem(mir_unit_t *mu, mir_type_t type);
 mir_type_t mir_get_pointer(mir_unit_t *mu, mir_type_t type);
 unsigned mir_get_dims(mir_unit_t *mu, mir_type_t type);
 unsigned mir_get_size(mir_unit_t *mu, mir_type_t type);
+bool mir_get_signed(mir_unit_t *mu, mir_type_t type);
 mir_class_t mir_get_class(mir_unit_t *mu, mir_type_t type);
 mir_repr_t mir_get_repr(mir_unit_t *mu, mir_type_t type);
 const mir_type_t *mir_get_fields(mir_unit_t *mu, mir_type_t type,
@@ -517,6 +531,11 @@ mir_value_t mir_build_binary(mir_unit_t *mu, mir_vec_op_t op, mir_type_t type,
                              mir_value_t left, mir_value_t right);
 mir_value_t mir_build_unary(mir_unit_t *mu, mir_vec_op_t op, mir_type_t type,
                             mir_value_t arg);
+mir_value_t mir_build_insert(mir_unit_t *mu, mir_value_t part, mir_value_t full,
+                             mir_value_t pos);
+mir_value_t mir_build_extract(mir_unit_t *mu, mir_type_t type, mir_value_t full,
+                              mir_value_t pos);
+mir_value_t mir_build_test(mir_unit_t *mu, mir_value_t vec);
 
 // Memory
 void mir_build_store(mir_unit_t *mu, mir_value_t dest, mir_value_t src);
@@ -562,7 +581,7 @@ void mir_build_case(mir_unit_t *mu, mir_value_t value, mir_block_t def,
                     const mir_value_t *cases, const mir_block_t *blocks,
                     int ncases);
 void mir_build_return(mir_unit_t *mu, mir_value_t value);
-void mir_build_wait(mir_unit_t *mu, mir_block_t target, mir_value_t time);
+void mir_build_wait(mir_unit_t *mu, mir_block_t target);
 void mir_build_consume(mir_unit_t *mu, mir_value_t value);
 mir_value_t mir_build_fcall(mir_unit_t *mu, ident_t name, mir_type_t type,
                             mir_stamp_t stamp, const mir_value_t *args,
@@ -625,8 +644,13 @@ void mir_build_drive_signal(mir_unit_t *mu, mir_value_t target,
 void mir_build_sched_waveform(mir_unit_t *mu, mir_value_t target,
                               mir_value_t count, mir_value_t values,
                               mir_value_t reject, mir_value_t after);
+void mir_build_put_driver(mir_unit_t *mu, mir_value_t target,
+                          mir_value_t count, mir_value_t values);
 void mir_build_deposit_signal(mir_unit_t *mu, mir_value_t target,
                               mir_value_t count, mir_value_t values);
+void mir_build_sched_deposit(mir_unit_t *mu, mir_value_t target,
+                             mir_value_t count, mir_value_t values,
+                             mir_value_t after);
 mir_value_t mir_build_resolved(mir_unit_t *mu, mir_value_t signal);
 mir_value_t mir_build_last_value(mir_unit_t *mu, mir_value_t signal);
 mir_value_t mir_build_driving_value(mir_unit_t *mu, mir_value_t signal,
@@ -681,6 +705,8 @@ void mir_build_map_implicit(mir_unit_t *mu, mir_value_t src, mir_value_t dst,
                             mir_value_t count);
 
 // Triggers
+mir_value_t mir_build_level_trigger(mir_unit_t *mu, mir_value_t signal,
+                                    mir_value_t count);
 mir_value_t mir_build_cmp_trigger(mir_unit_t *mu, mir_value_t left,
                                   mir_value_t right);
 mir_value_t mir_build_function_trigger(mir_unit_t *mu, ident_t name,
@@ -703,10 +729,10 @@ mir_value_t mir_build_context_upref(mir_unit_t *mu, int hops);
 mir_value_t mir_build_var_upref(mir_unit_t *mu, int hops, int nth);
 
 // Events
-void mir_build_sched_event(mir_unit_t *mu, mir_value_t signal,
-                           mir_value_t count);
-void mir_build_clear_event(mir_unit_t *mu, mir_value_t signal,
-                           mir_value_t count);
+void mir_build_sched_event(mir_unit_t *mu, mir_value_t on, mir_value_t count);
+void mir_build_clear_event(mir_unit_t *mu, mir_value_t on, mir_value_t count);
+void mir_build_sched_process(mir_unit_t *mu, mir_value_t delay);
+void mir_build_sched_inactive(mir_unit_t *mu);
 
 // Reflection
 mir_value_t mir_build_reflect_value(mir_unit_t *mu, mir_value_t value,
