@@ -592,12 +592,15 @@ static void gvn_visit_block(mir_unit_t *mu, mir_block_t block,
       case MIR_OP_CAST:
       case MIR_OP_PACK:
       case MIR_OP_VAR_UPREF:
+      case MIR_OP_CONST:
+      case MIR_OP_CONST_REAL:
       case MIR_OP_CONST_VEC:
       case MIR_OP_BINARY:
       case MIR_OP_UNARY:
       case MIR_OP_CLOSURE:
       case MIR_OP_CONTEXT_UPREF:
       case MIR_OP_UNWRAP:
+      case MIR_OP_INSERT:
          gvn_generic(mu, node, block, opt);
          break;
       case MIR_OP_AND:
