@@ -289,13 +289,7 @@ static void import_report(mir_unit_t *mu, mir_import_t *imp, int op)
 static void import_wait(mir_unit_t *mu, mir_import_t *imp, int op)
 {
    mir_block_t btarget = imp->blocks[vcode_get_target(op, 0)];
-
-   mir_value_t after = MIR_NULL_VALUE;
-   vcode_reg_t after_reg = vcode_get_arg(op, 0);
-   if (after_reg != VCODE_INVALID_REG)
-      after = imp->map[after_reg];
-
-   mir_build_wait(mu, btarget, after);
+   mir_build_wait(mu, btarget);
 }
 
 static void import_jump(mir_unit_t *mu, mir_import_t *imp, int op)
@@ -1301,15 +1295,6 @@ static void import_enter_state(mir_unit_t *mu, mir_import_t *imp, int op)
    mir_build_enter_state(mu, state, strong);
 }
 
-static void import_deposit_signal(mir_unit_t *mu, mir_import_t *imp, int op)
-{
-   mir_value_t target = imp->map[vcode_get_arg(op, 0)];
-   mir_value_t count = imp->map[vcode_get_arg(op, 1)];
-   mir_value_t values = imp->map[vcode_get_arg(op, 2)];
-
-   mir_build_deposit_signal(mu, target, count, values);
-}
-
 static void import_syscall(mir_unit_t *mu, mir_import_t *imp, int op)
 {
    mir_value_t locus = imp->map[vcode_get_arg(op, 0)];
@@ -1362,6 +1347,18 @@ static void import_reflect_subtype(mir_unit_t *mu, mir_import_t *imp, int op)
       mir_build_reflect_subtype(mu, context, locus, bounds);
 }
 
+static void import_debug_out(mir_unit_t *mu, mir_import_t *imp, int op)
+{
+   mir_value_t value = imp->map[vcode_get_arg(op, 0)];
+   mir_build_debug_out(mu, value);
+}
+
+static void import_sched_process(mir_unit_t *mu, mir_import_t *imp, int op)
+{
+   mir_value_t delay = imp->map[vcode_get_arg(op, 0)];
+   mir_build_sched_process(mu, delay);
+}
+
 static void import_block(mir_unit_t *mu, mir_import_t *imp)
 {
    const int nops = vcode_count_ops();
@@ -1754,9 +1751,6 @@ static void import_block(mir_unit_t *mu, mir_import_t *imp)
       case VCODE_OP_ENTER_STATE:
          import_enter_state(mu, imp, i);
          break;
-      case VCODE_OP_DEPOSIT_SIGNAL:
-         import_deposit_signal(mu, imp, i);
-         break;
       case VCODE_OP_SYSCALL:
          import_syscall(mu, imp, i);
          break;
@@ -1766,6 +1760,12 @@ static void import_block(mir_unit_t *mu, mir_import_t *imp)
       case VCODE_OP_REFLECT_VALUE:
          import_reflect_value(mu, imp, i);
          break;
+      case VCODE_OP_DEBUG_OUT:
+         import_debug_out(mu, imp, i);
+         break;
+      case VCODE_OP_SCHED_PROCESS:
+         import_sched_process(mu, imp, i);
+         break;
       default:
          vcode_dump_with_mark(i, NULL, NULL);
          fatal_trace("cannot import vcode op %s",
