@@ -19,14 +19,14 @@
 #define _SCAN_H
 
 #include "prim.h"
-
-typedef struct { ident_t left, right; } ident_pair_t;
+#include "diag.h"
 
 typedef union {
-   double   real;
-   char    *str;
-   int64_t  i64;
-   ident_t  ident;
+   double      real;
+   char       *str;
+   int64_t     i64;
+   ident_t     ident;
+   text_buf_t *text;
 } yylval_t;
 
 // Functions shared between VHDL and Verilog scanners
@@ -39,8 +39,24 @@ typedef enum {
 
 typedef int token_t;
 
+typedef enum {
+   VLOG_1364_1995,
+   VLOG_1364_2001_NOCONFIG,
+   VLOG_1364_2001,
+   VLOG_1364_2005,
+   VLOG_1800_2005,
+   VLOG_1800_2009,
+   VLOG_1800_2012,
+   VLOG_1800_2017,
+   VLOG_1800_2023,
+} vlog_version_t;
+
 void input_from_file(const char *file);
-void input_from_buffer(const char *buf, size_t len, hdl_kind_t hdl);
+void input_from_buffer(const char *buf, size_t len, file_ref_t file_ref,
+                       hdl_kind_t hdl);
+void push_buffer(const char *buf, size_t len, file_ref_t file_ref);
+void push_file(const char *file, const loc_t *srcloc);
+void pop_buffer(void);
 hdl_kind_t source_kind(void);
 token_t processed_yylex(void);
 const char *token_str(token_t tok);
@@ -52,6 +68,8 @@ void pp_defines_add(const char *name, const char *value);
 const char *pp_defines_get(const char *name);
 void pp_defines_iter(pp_iter_t fn, void *ctx);
 
+void add_include_dir(const char *str);
+
 void scan_as_psl(void);
 void scan_as_vhdl(void);
 void scan_as_verilog(void);
@@ -59,6 +77,14 @@ void scan_as_udp(void);
 void scan_as_sdf(void);
 void scan_as_sdf_expr(void);
 
+const char *verilog_version_string(vlog_version_t vers);
+bool parse_verilog_version(const char *str, vlog_version_t *vers);
+
+void set_default_keywords(vlog_version_t vers);
+void push_keywords(vlog_version_t vers);
+bool pop_keywords(void);
+bool get_verilog_keywords(vlog_version_t *vers);
+
 // Private interface to Flex scanners
 
 void begin_token(char *tok, int length);
@@ -257,7 +283,7 @@ void reset_sdf_parser(void);
 #define tNEGEDGE       356
 #define tINITIAL       357
 #define tWIRE          358
-#define tUNSIGNED      359
+#define tUNSNUM        359
 #define tASSUME        360
 #define tASSUMEG       361
 #define tRESTRICT      362
@@ -478,5 +504,41 @@ void reset_sdf_parser(void);
 #define tLARGE         577
 #define tVECTORED      578
 #define tSCALARED      579
+#define tUNCTDRIVE     580
+#define tNOUNCTDRIVE   581
+#define tDEASSIGN      582
+#define tSIGNED        583
+#define tUNSIGNED      584
+#define tDISABLE       585
+#define tCLASS         586
+#define tCHANDLE       587
+#define tEXPORT        588
+#define tIMPORT        589
+#define tENDPACKAGE    590
+#define tEXTENDS       591
+#define tTHIS          592
+#define tNETTYPE       593
+#define tREF           594
+#define tSUPER         595
+#define tSTATIC        596
+#define tPLUSEQ        597
+#define tMINUSEQ       598
+#define tTIMESEQ       599
+#define tDIVEQ         600
+#define tPERCENTEQ     601
+#define tAMPEQ         602
+#define tBAREQ         603
+#define tCARETEQ       604
+#define tLSLEQ         605
+#define tLSREQ         606
+#define tASLEQ         607
+#define tASREQ         608
+#define tBUFIF0        609
+#define tBUFIF1        610
+#define tNOTIF0        611
+#define tNOTIF1        612
+#define tPROTECT       613
+#define tBEGINPROT     614
+#define tENDPROT       615
 
 #endif  // _SCAN_H
