@@ -165,7 +165,6 @@ typedef enum {
    VCODE_OP_OR_TRIGGER,
    VCODE_OP_CMP_TRIGGER,
    VCODE_OP_INSTANCE_NAME,
-   VCODE_OP_DEPOSIT_SIGNAL,
    VCODE_OP_MAP_IMPLICIT,
    VCODE_OP_BIND_EXTERNAL,
    VCODE_OP_ARRAY_SCOPE,
@@ -173,6 +172,7 @@ typedef enum {
    VCODE_OP_SYSCALL,
    VCODE_OP_PUT_CONVERSION,
    VCODE_OP_DIR_CHECK,
+   VCODE_OP_SCHED_PROCESS,
 } vcode_op_t;
 
 typedef enum {
@@ -399,7 +399,7 @@ void emit_pcall(ident_t func, const vcode_reg_t *args, int nargs,
                 vcode_block_t resume_bb);
 vcode_reg_t emit_syscall(ident_t func, vcode_type_t type, vcode_type_t bounds,
                          vcode_reg_t locus, const vcode_reg_t *args, int nargs);
-void emit_wait(vcode_block_t target, vcode_reg_t time);
+void emit_wait(vcode_block_t target);
 void emit_jump(vcode_block_t target);
 vcode_reg_t emit_load(vcode_var_t var);
 vcode_reg_t emit_load_indirect(vcode_reg_t reg);
@@ -459,6 +459,7 @@ vcode_reg_t emit_array_ref(vcode_reg_t array, vcode_reg_t offset);
 void emit_copy(vcode_reg_t dest, vcode_reg_t src, vcode_reg_t count);
 void emit_sched_event(vcode_reg_t nets, vcode_reg_t n_elems);
 void emit_clear_event(vcode_reg_t nets, vcode_reg_t count);
+void emit_sched_process(vcode_reg_t delay);
 void emit_resume(ident_t func);
 void emit_memset(vcode_reg_t ptr, vcode_reg_t value, vcode_reg_t len);
 void emit_case(vcode_reg_t value, vcode_block_t def, const vcode_reg_t *cases,
@@ -533,8 +534,6 @@ void emit_convert_in(vcode_reg_t conv, vcode_reg_t nets, vcode_reg_t count);
 void emit_convert_out(vcode_reg_t conv, vcode_reg_t nets, vcode_reg_t count);
 void emit_bind_foreign(vcode_reg_t spec, vcode_reg_t length, vcode_reg_t locus);
 vcode_reg_t emit_instance_name(vcode_reg_t kind);
-void emit_deposit_signal(vcode_reg_t signal, vcode_reg_t count,
-                         vcode_reg_t values);
 vcode_reg_t emit_bind_external(vcode_reg_t locus, ident_t scope,
                                vcode_type_t type, vcode_type_t bounds);
 void emit_put_conversion(vcode_reg_t cf, vcode_reg_t target, vcode_reg_t count,
