@@ -60,6 +60,8 @@ static void vlog_dump_paren(vlog_node_t v, int indent)
    case V_REF:
    case V_BIT_SELECT:
    case V_NUMBER:
+   case V_USER_FCALL:
+   case V_SYS_FCALL:
       vlog_dump(v, indent);
       break;
    default:
@@ -140,7 +142,17 @@ static void vlog_dump_udp_entry(vlog_node_t v, int indent)
          last = kind;
       }
 
-      print_syntax("%c", (char)vlog_ival(sym));
+      switch (vlog_kind(sym)) {
+      case V_UDP_LEVEL:
+         print_syntax("%c", (char)vlog_ival(sym));
+         break;
+      case V_UDP_EDGE:
+         print_syntax("(%c%c)", (char)vlog_ival(vlog_left(sym)),
+                      (char)vlog_ival(vlog_right(sym)));
+         break;
+      default:
+         should_not_reach_here();
+      }
    }
 
    print_syntax(";\n");
@@ -223,7 +235,13 @@ static void vlog_dump_always(vlog_node_t v, int indent)
 {
    tab(indent);
 
-   print_syntax("#always ");
+   switch (vlog_subkind(v)) {
+   case V_ALWAYS_FF:    print_syntax("#always_ff "); break;
+   case V_ALWAYS_COMB:  print_syntax("#always_comb "); break;
+   case V_ALWAYS_LATCH: print_syntax("#always_latch "); break;
+   default:             print_syntax("#always "); break;
+   }
+
    vlog_dump(vlog_stmt(v, 0), indent);
    print_syntax("\n");
 }
@@ -260,10 +278,12 @@ static void vlog_dump_timing(vlog_node_t v, int indent)
 {
    vlog_dump(vlog_value(v), indent);
 
-   if (vlog_stmts(v) > 0)
+   if (vlog_stmts(v) > 0) {
+      print_syntax(" ");
       vlog_dump(vlog_stmt(v, 0), indent);
+   }
    else
-      print_syntax(";\n");
+      print_syntax(";");
 }
 
 static void vlog_dump_event(vlog_node_t v)
@@ -286,15 +306,17 @@ static void vlog_dump_event_control(vlog_node_t v)
       vlog_dump(vlog_param(v, i), 0);
    }
 
-   print_syntax(") ");
+   print_syntax(")");
 }
 
 static void vlog_dump_nbassign(vlog_node_t v, int indent)
 {
    vlog_dump(vlog_target(v), 0);
    print_syntax(" <= ");
-   if (vlog_has_delay(v))
+   if (vlog_has_delay(v)) {
       vlog_dump(vlog_delay(v), 0);
+      print_syntax(" ");
+   }
    vlog_dump(vlog_value(v), 0);
    print_syntax(";");
 }
@@ -302,15 +324,12 @@ static void vlog_dump_nbassign(vlog_node_t v, int indent)
 static void vlog_dump_bassign(vlog_node_t v, int indent)
 {
    vlog_dump(vlog_target(v), 0);
+   print_syntax(" = ");
 
-   switch (vlog_subkind(v)) {
-   case V_ASSIGN_EQUALS:
-      print_syntax(" = ");
-      break;
-   }
-
-   if (vlog_has_delay(v))
+   if (vlog_has_delay(v)) {
       vlog_dump(vlog_delay(v), 0);
+      print_syntax(" ");
+   }
 
    vlog_dump(vlog_value(v), 0);
    print_syntax(";");
@@ -395,8 +414,13 @@ static void vlog_dump_string(vlog_node_t v)
    print_syntax("\"");
 
    number_t n = vlog_number(v);
-   for (int i = number_width(n)/8 - 1; i >= 0; i--)
-      print_syntax("%c", number_byte(n, i));
+   for (int i = number_width(n)/8 - 1; i >= 0; i--) {
+      const uint8_t byte = number_byte(n, i);
+      if (byte >= 0x20 && byte <= 0x7e)
+         print_syntax("%c", byte);
+      else
+         print_syntax("\\x%02x", byte);
+   }
 
    print_syntax("\"");
 }
@@ -421,6 +445,7 @@ static void vlog_dump_binary(vlog_node_t v)
    switch (vlog_subkind(v)) {
    case V_BINARY_OR:       print_syntax(" | "); break;
    case V_BINARY_AND:      print_syntax(" & "); break;
+   case V_BINARY_XOR:      print_syntax(" ^ "); break;
    case V_BINARY_CASE_EQ:  print_syntax(" === "); break;
    case V_BINARY_CASE_NEQ: print_syntax(" !== "); break;
    case V_BINARY_LOG_EQ:   print_syntax(" == "); break;
@@ -429,8 +454,14 @@ static void vlog_dump_binary(vlog_node_t v)
    case V_BINARY_LOG_AND:  print_syntax(" && "); break;
    case V_BINARY_PLUS:     print_syntax(" + "); break;
    case V_BINARY_MINUS:    print_syntax(" - "); break;
+   case V_BINARY_TIMES:    print_syntax(" * "); break;
+   case V_BINARY_DIVIDE:   print_syntax(" / "); break;
    case V_BINARY_LT:       print_syntax(" < "); break;
    case V_BINARY_GT:       print_syntax(" > "); break;
+   case V_BINARY_SHIFT_LL: print_syntax(" << "); break;
+   case V_BINARY_SHIFT_RL: print_syntax(" >> "); break;
+   case V_BINARY_SHIFT_LA: print_syntax(" <<< "); break;
+   case V_BINARY_SHIFT_RA: print_syntax(" >>> "); break;
    }
 
    vlog_dump_paren(vlog_right(v), 0);
@@ -443,7 +474,7 @@ static void vlog_dump_unary(vlog_node_t v)
    case V_UNARY_BITNEG: print_syntax("~~"); break;
    }
 
-   vlog_dump(vlog_value(v), 0);
+   vlog_dump_paren(vlog_value(v), 0);
 }
 
 static void vlog_dump_postfix(vlog_node_t v)
@@ -466,11 +497,23 @@ static void vlog_dump_prefix(vlog_node_t v)
    vlog_dump(vlog_target(v), 0);
 }
 
+static void vlog_dump_user_fcall(vlog_node_t v)
+{
+   print_syntax("%s(", istr(vlog_ident(v)));
+
+   const int nparams = vlog_params(v);
+   for (int i = 0; i < nparams; i++) {
+      if (i > 0) print_syntax(", ");
+      vlog_dump(vlog_param(v, i), 0);
+   }
+
+   print_syntax(")");
+}
+
 static void vlog_dump_delay_control(vlog_node_t v, int indent)
 {
    print_syntax("##");
    vlog_dump(vlog_value(v), 0);
-   print_syntax(" ");
 }
 
 static void vlog_dump_gate_inst(vlog_node_t v, int indent)
@@ -592,6 +635,7 @@ static void vlog_dump_data_type(vlog_node_t v, int indent)
    case DT_INTEGER: print_syntax("#integer"); break;
    case DT_BIT: print_syntax("#bit"); break;
    case DT_INT: print_syntax("#int"); break;
+   case DT_BYTE: print_syntax("#byte"); break;
    default: should_not_reach_here();
    }
 
@@ -776,6 +820,14 @@ static void vlog_dump_ref(vlog_node_t v, int indent)
    print_syntax("%s", istr(vlog_ident(v)));
 }
 
+static void vlog_dump_hier_ref(vlog_node_t v, int indent)
+{
+   if (vlog_has_ref(v))
+      vlog_dump_address(vlog_ref(v));
+
+   print_syntax("%s.%s", istr(vlog_ident2(v)), istr(vlog_ident(v)));
+}
+
 static void vlog_dump_task_decl(vlog_node_t v, int indent)
 {
    tab(indent);
@@ -797,9 +849,20 @@ static void vlog_dump_task_decl(vlog_node_t v, int indent)
 static void vlog_dump_func_decl(vlog_node_t v, int indent)
 {
    tab(indent);
-   print_syntax("#function %s(", istr(vlog_ident(v)));
+   print_syntax("#function %s;", istr(vlog_ident(v)));
 
-   print_syntax(");\n");
+   if (vlog_has_ident2(v))
+      print_syntax(" -- %s.%s", istr(vlog_ident2(v)), istr(vlog_ident(v)));
+
+   print_syntax("\n");
+
+   const int nports = vlog_ports(v);
+   for (int i = 0; i < nports; i++)
+      vlog_dump(vlog_port(v, i), indent + 2);
+
+   const int ndecls = vlog_decls(v);
+   for (int i = 0; i < ndecls; i++)
+      vlog_dump(vlog_decl(v, i), indent + 2);
 
    const int nstmts = vlog_stmts(v);
    for (int i = 0; i < nstmts; i++) {
@@ -820,6 +883,49 @@ static void vlog_dump_wait(vlog_node_t v, int indent)
    vlog_dump_stmt_or_null(v, indent);
 }
 
+static void vlog_dump_op_assign(vlog_node_t v)
+{
+   vlog_dump(vlog_target(v), 0);
+
+   switch (vlog_subkind(v)) {
+   case V_ASSIGN_EQUALS: print_syntax(" = "); break;
+   case V_ASSIGN_PLUS:   print_syntax(" += "); break;
+   }
+
+   vlog_dump(vlog_value(v), 0);
+   print_syntax(";");
+}
+
+static void vlog_dump_enum_decl(vlog_node_t v)
+{
+   print_syntax("#enum");
+
+   if (vlog_has_type(v)) {
+      print_syntax(" ");
+      vlog_dump(vlog_type(v), 0);
+   }
+
+   print_syntax(" { ");
+
+   const int ndecls = vlog_decls(v);
+   for (int i = 0; i < ndecls; i++) {
+      if (i > 0) print_syntax(", ");
+      vlog_dump(vlog_decl(v, i), 0);
+   }
+
+   print_syntax(" }");
+}
+
+static void vlog_dump_enum_name(vlog_node_t v)
+{
+   print_syntax("%s", istr(vlog_ident(v)));
+
+   if (vlog_has_value(v)) {
+      print_syntax("=");
+      vlog_dump(vlog_value(v), 0);
+   }
+}
+
 void vlog_dump(vlog_node_t v, int indent)
 {
    switch (vlog_kind(v)) {
@@ -835,7 +941,11 @@ void vlog_dump(vlog_node_t v, int indent)
    case V_REF:
       vlog_dump_ref(v, indent);
       break;
+   case V_HIER_REF:
+      vlog_dump_hier_ref(v, indent);
+      break;
    case V_PORT_DECL:
+   case V_TF_PORT_DECL:
       vlog_dump_port_decl(v, indent);
       break;
    case V_NET_DECL:
@@ -984,6 +1094,18 @@ void vlog_dump(vlog_node_t v, int indent)
    case V_WAIT:
       vlog_dump_wait(v, indent);
       break;
+   case V_USER_FCALL:
+      vlog_dump_user_fcall(v);
+      break;
+   case V_OP_ASSIGN:
+      vlog_dump_op_assign(v);
+      break;
+   case V_ENUM_DECL:
+      vlog_dump_enum_decl(v);
+      break;
+   case V_ENUM_NAME:
+      vlog_dump_enum_name(v);
+      break;
    default:
       print_syntax("\n");
       fflush(stdout);
