@@ -142,6 +142,17 @@ typedef enum {
    V_VOID_CALL,
    V_GENVAR_DECL,
    V_FOR_GENERATE,
+   V_DEASSIGN,
+   V_FORCE,
+   V_RELEASE,
+   V_DISABLE,
+   V_HIER_REF,
+   V_TF_PORT_DECL,
+   V_RETURN,
+   V_OP_ASSIGN,
+   V_STRUCT_REF,
+   V_PACKAGE,
+   V_MIN_TYP_MAX,
 
    V_LAST_NODE_KIND
 } vlog_kind_t;
@@ -198,6 +209,7 @@ typedef enum {
    V_BINARY_EXP,
    V_BINARY_XOR,
    V_BINARY_XNOR,
+   V_BINARY_NAND,
 } vlog_binary_t;
 
 typedef enum {
@@ -208,6 +220,9 @@ typedef enum {
    V_UNARY_AND,
    V_UNARY_OR,
    V_UNARY_XOR,
+   V_UNARY_NAND,
+   V_UNARY_NOR,
+   V_UNARY_XNOR,
 } vlog_unary_t;
 
 typedef enum {
@@ -217,6 +232,18 @@ typedef enum {
 
 typedef enum {
    V_ASSIGN_EQUALS,
+   V_ASSIGN_PLUS,
+   V_ASSIGN_MINUS,
+   V_ASSIGN_TIMES,
+   V_ASSIGN_DIVIDE,
+   V_ASSIGN_MOD,
+   V_ASSIGN_AND,
+   V_ASSIGN_OR,
+   V_ASSIGN_XOR,
+   V_ASSIGN_SHIFT_LL,
+   V_ASSIGN_SHIFT_RL,
+   V_ASSIGN_SHIFT_LA,
+   V_ASSIGN_SHIFT_RA,
 } vlog_assign_t;
 
 typedef enum {
@@ -229,7 +256,11 @@ typedef enum {
    V_GATE_XOR,
    V_GATE_XNOR,
    V_GATE_NOT,
+   V_GATE_NOTIF0,
+   V_GATE_NOTIF1,
    V_GATE_BUF,
+   V_GATE_BUFIF0,
+   V_GATE_BUFIF1,
 } vlog_gate_kind_t;
 
 typedef enum {
@@ -272,6 +303,7 @@ bool vlog_has_ident(vlog_node_t v);
 
 ident_t vlog_ident2(vlog_node_t v);
 void vlog_set_ident2(vlog_node_t v, ident_t i);
+bool vlog_has_ident2(vlog_node_t v);
 
 vlog_node_t vlog_ref(vlog_node_t v);
 void vlog_set_ref(vlog_node_t v, vlog_node_t d);
