@@ -42,47 +42,18 @@ static bool has_error(vlog_node_t v)
 {
    switch (vlog_kind(v)) {
    case V_REF:
-   case V_USER_FCALL:
-   case V_USER_TCALL:
       if (vlog_has_ref(v))
          return false;
       else {
          assert(error_count() > 0);
          return true;
       }
+   case V_USER_FCALL:
+   case V_USER_TCALL:
+   case V_HIER_REF:
+      // May not have reported error yet
+      return !vlog_has_ref(v);
    case V_PORT_DECL:
-      if (vlog_has_ref(v))
-         return false;
-      else
-         return true; // XXX: check this
-   default:
-      return false;
-   }
-}
-
-static bool constant_equal(vlog_node_t a, vlog_node_t b)
-{
-   const vlog_kind_t kind = vlog_kind(a);
-   if (kind != vlog_kind(b))
-      return false;
-
-   if (has_error(a) || has_error(b))
-      return true;   // Suppress cascading errors
-
-   switch (kind) {
-   case V_NUMBER:
-      {
-         number_t an = vlog_number(a);
-         number_t bn = vlog_number(b);
-
-         return number_equal(an, bn);
-      }
-   case V_BINARY:
-      return vlog_subkind(a) == vlog_subkind(b)
-         && constant_equal(vlog_left(a), vlog_left(b))
-         && constant_equal(vlog_right(a), vlog_right(b));
-   case V_REF:
-      return vlog_ref(a) == vlog_ref(b);
    default:
       return false;
    }
@@ -99,6 +70,8 @@ static void vlog_check_const_expr(vlog_node_t expr)
          switch (vlog_kind(decl)) {
          case V_PARAM_DECL:
          case V_LOCALPARAM:
+         case V_GENVAR_DECL:
+         case V_ENUM_NAME:
             break;
          default:
             {
@@ -137,26 +110,68 @@ static void vlog_check_const_expr(vlog_node_t expr)
    }
 }
 
-static void vlog_check_variable_target(vlog_node_t target)
+static void vlog_check_variable_lvalue(vlog_node_t v, vlog_node_t where)
 {
-   if (vlog_is_net(target)) {
-      diag_t *d = diag_new(DIAG_ERROR, vlog_loc(target));
-      name_for_diag(d, target, "target");
-      diag_printf(d, " cannot be assigned in a procedural block");
-      diag_emit(d);
+   switch (vlog_kind(v)) {
+   case V_VAR_DECL:
+   case V_FUNC_DECL:
+   case V_STRUCT_DECL:
+   case V_ENUM_DECL:
+   case V_UNION_DECL:
+   case V_GENVAR_DECL:
+      break;
+   case V_REF:
+      if (vlog_has_ref(v))
+         vlog_check_variable_lvalue(vlog_ref(v), v);
+      break;
+   case V_BIT_SELECT:
+   case V_PART_SELECT:
+   case V_STRUCT_REF:
+      vlog_check_variable_lvalue(vlog_value(v), v);
+      break;
+   case V_CONCAT:
+      {
+         const int nparams = vlog_params(v);
+         for (int i = 0; i < nparams; i++) {
+            vlog_node_t p = vlog_param(v, i);
+            vlog_check_variable_lvalue(p, p);
+         }
+      }
+      break;
+   case V_PORT_DECL:
+      if (vlog_has_ref(v)) {
+         vlog_check_variable_lvalue(vlog_ref(v), where);
+         break;
+      }
+      // Fall-through
+   default:
+      {
+         diag_t *d = diag_new(DIAG_ERROR, vlog_loc(where));
+         name_for_diag(d, where, "target");
+         diag_suppress(d, has_error(where));
+         diag_printf(d, " cannot be assigned in a procedural block");
+         diag_emit(d);
+      }
+      break;
    }
 }
 
 static void vlog_check_nbassign(vlog_node_t stmt)
 {
    vlog_node_t target = vlog_target(stmt);
-   vlog_check_variable_target(target);
+   vlog_check_variable_lvalue(target, target);
 }
 
 static void vlog_check_bassign(vlog_node_t stmt)
 {
    vlog_node_t target = vlog_target(stmt);
-   vlog_check_variable_target(target);
+   vlog_check_variable_lvalue(target, target);
+}
+
+static void vlog_check_op_assign(vlog_node_t stmt)
+{
+   vlog_node_t target = vlog_target(stmt);
+   vlog_check_variable_lvalue(target, target);
 }
 
 static void vlog_check_net_lvalue(vlog_node_t v, vlog_node_t where)
@@ -193,6 +208,7 @@ static void vlog_check_net_lvalue(vlog_node_t v, vlog_node_t where)
          diag_printf(d, " cannot be driven by continuous assignment");
          diag_emit(d);
       }
+      break;
    }
 }
 
@@ -223,7 +239,7 @@ static void vlog_check_consistent(vlog_node_t a, vlog_node_t b)
       vlog_node_t aright = vlog_right(ar);
       vlog_node_t bright = vlog_right(br);
 
-      if (!constant_equal(aleft, bleft) || !constant_equal(aright, bright)) {
+      if (!vlog_equal_node(aleft, bleft) || !vlog_equal_node(aright, bright)) {
          diag_t *d = diag_new(DIAG_ERROR, vlog_loc(br));
          diag_printf(d, "inconsistent dimensions for '%s'",
                      istr(vlog_ident(b)));
@@ -235,7 +251,8 @@ static void vlog_check_consistent(vlog_node_t a, vlog_node_t b)
 
 static void vlog_check_port_decl(vlog_node_t port)
 {
-   vlog_check_consistent(port, vlog_ref(port));
+   if (vlog_has_ref(port))
+      vlog_check_consistent(port, vlog_ref(port));
 }
 
 static void vlog_check_primitive(vlog_node_t udp)
@@ -389,6 +406,60 @@ static void vlog_check_user_tcall(vlog_node_t v)
    vlog_check_call_args(v, func);
 }
 
+static void vlog_check_deassign(vlog_node_t v)
+{
+   error_at(vlog_loc(v), "procedural deassign statements are not supported "
+            "as they are being considered for removal from the System Verilog "
+            "standard");
+}
+
+static void vlog_check_return(vlog_node_t v)
+{
+   if (!vlog_has_ref(v))
+      return;   // Was earlier error
+
+   vlog_node_t subr = vlog_ref(v);
+   const vlog_kind_t kind = vlog_kind(subr);
+
+   if (kind == V_FUNC_DECL && !vlog_has_value(v))
+      error_at(vlog_loc(v), "return statement in a non-void function must "
+               "have an expression");
+   else if (kind == V_TASK_DECL && vlog_has_value(v))
+      error_at(vlog_loc(v), "return statement in a task cannot have an "
+               "expression");
+}
+
+static void vlog_check_concat(vlog_node_t v)
+{
+   if (vlog_has_value(v))
+      vlog_check_const_expr(vlog_value(v));
+}
+
+static void vlog_check_module(vlog_node_t v)
+{
+   const int nports = vlog_ports(v);
+   for (int i = 0; i < nports; i++) {
+      vlog_node_t ref = vlog_port(v, i);
+      if (!vlog_has_ref(ref))
+         error_at(vlog_loc(ref), "missing port declaration for '%s'",
+                  istr(vlog_ident(ref)));
+   }
+}
+
+static void vlog_check_enum_name(vlog_node_t v)
+{
+   if (vlog_has_value(v))
+      vlog_check_const_expr(vlog_value(v));
+}
+
+static void vlog_check_hier_ref(vlog_node_t v)
+{
+   vlog_node_t inst = vlog_ref(v);
+   if (vlog_kind(inst) != V_MOD_INST)
+      error_at(vlog_loc(v), "prefix of hierarchical identifier is not an "
+               "instance");
+}
+
 static vlog_node_t vlog_check_cb(vlog_node_t v, void *ctx)
 {
    if (has_error(v))
@@ -396,6 +467,7 @@ static vlog_node_t vlog_check_cb(vlog_node_t v, void *ctx)
 
    switch (vlog_kind(v)) {
    case V_MODULE:
+      vlog_check_module(v);
       break;
    case V_PRIMITIVE:
       vlog_check_primitive(v);
@@ -406,6 +478,9 @@ static vlog_node_t vlog_check_cb(vlog_node_t v, void *ctx)
    case V_BASSIGN:
       vlog_check_bassign(v);
       break;
+   case V_OP_ASSIGN:
+      vlog_check_op_assign(v);
+      break;
    case V_ASSIGN:
       vlog_check_assign(v);
       break;
@@ -433,13 +508,27 @@ static vlog_node_t vlog_check_cb(vlog_node_t v, void *ctx)
    case V_USER_TCALL:
       vlog_check_user_tcall(v);
       break;
+   case V_DEASSIGN:
+      vlog_check_deassign(v);
+      break;
+   case V_RETURN:
+      vlog_check_return(v);
+      break;
+   case V_CONCAT:
+      vlog_check_concat(v);
+      break;
+   case V_ENUM_NAME:
+      vlog_check_enum_name(v);
+      break;
+   case V_HIER_REF:
+      vlog_check_hier_ref(v);
+      break;
    case V_CASE_ITEM:
    case V_UDP_LEVEL:
    case V_UDP_EDGE:
    case V_UDP_ENTRY:
    case V_UDP_TABLE:
    case V_GATE_INST:
-   case V_ENUM_NAME:
    case V_ENUM_DECL:
    case V_STRUCT_DECL:
    case V_UNION_DECL:
@@ -460,7 +549,6 @@ static vlog_node_t vlog_check_cb(vlog_node_t v, void *ctx)
    case V_TASK_DECL:
    case V_EMPTY:
    case V_COND_EXPR:
-   case V_CONCAT:
    case V_FOR_LOOP:
    case V_FOR_INIT:
    case V_FOR_STEP:
@@ -490,6 +578,12 @@ static vlog_node_t vlog_check_cb(vlog_node_t v, void *ctx)
    case V_VOID_CALL:
    case V_GENVAR_DECL:
    case V_FOR_GENERATE:
+   case V_FORCE:
+   case V_RELEASE:
+   case V_TF_PORT_DECL:
+   case V_WHILE:
+   case V_DO_WHILE:
+   case V_STRUCT_REF:
       break;
    default:
       fatal_at(vlog_loc(v), "cannot check verilog node %s",
