@@ -110,55 +110,125 @@ static tree_t trans_expr(trans_gen_t *g, vlog_node_t v)
    }
 }
 
+static type_t trans_sized_type(vlog_node_t decl, verilog_type_t packed_type,
+                               int width)
+{
+   type_t packed = verilog_type(packed_type);
+
+   tree_t c = tree_new(T_CONSTRAINT);
+   tree_set_subkind(c, C_INDEX);
+   tree_set_loc(c, vlog_loc(decl));
+
+   type_t std_int = std_type(NULL, STD_INTEGER);
+
+   tree_t left = tree_new(T_LITERAL);
+   tree_set_subkind(left, L_INT);
+   tree_set_type(left, std_int);
+   tree_set_ival(left, width - 1);
+
+   tree_t right = tree_new(T_LITERAL);
+   tree_set_subkind(right, L_INT);
+   tree_set_type(right, std_int);
+   tree_set_ival(right, 0);
+
+   tree_t r = tree_new(T_RANGE);
+   tree_set_subkind(r, RANGE_DOWNTO);
+   tree_set_left(r, left);
+   tree_set_right(r, right);
+   tree_set_loc(r, vlog_loc(decl));
+   tree_set_type(r, std_int);
+
+   tree_add_range(c, r);
+
+   type_t sub = type_new(T_SUBTYPE);
+   type_set_base(sub, packed);
+   type_set_constraint(sub, c);
+
+   return sub;
+}
+
 static type_t trans_type(trans_gen_t *g, vlog_node_t decl,
                          verilog_type_t scalar_type,
                          verilog_type_t packed_type)
 {
-   const int nranges = vlog_ranges(decl);
-   if (nranges > 0) {
-      type_t packed = verilog_type(packed_type);
+   // TODO: cache decl ^ scalar_type ^ packed_type
 
-      tree_t c = tree_new(T_CONSTRAINT);
-      tree_set_subkind(c, C_INDEX);
-      tree_set_loc(c, vlog_loc(decl));
+   switch (vlog_kind(decl)) {
+   case V_TYPE_DECL:
+   case V_ENUM_DECL:
+      return trans_type(g, vlog_type(decl), scalar_type, packed_type);
+   case V_DATA_TYPE:
+      break;
+   default:
+      CANNOT_HANDLE(decl);
+   }
 
-      for (int i = 0; i < nranges; i++) {
-         type_t index_type = index_type_of(packed, i);
-         vlog_node_t vr = vlog_range(decl, i);
+   switch (vlog_subkind(decl)) {
+   case DT_LOGIC:
+   case DT_BIT:
+      {
+         const int nranges = vlog_ranges(decl);
+         if (nranges == 0)
+            return verilog_type(scalar_type);
 
-         tree_t left = trans_expr(g, vlog_left(vr));
-         tree_t right = trans_expr(g, vlog_right(vr));
+         type_t packed = verilog_type(packed_type);
 
-         int64_t ileft, iright;
-         const bool left_is_const = folded_int(left, &ileft);
-         const bool right_is_const = folded_int(right, &iright);
+         tree_t c = tree_new(T_CONSTRAINT);
+         tree_set_subkind(c, C_INDEX);
+         tree_set_loc(c, vlog_loc(decl));
 
-         range_kind_t dir;
-         if (left_is_const && right_is_const)
-            dir = ileft < iright ? RANGE_TO : RANGE_DOWNTO;
-         else if (left_is_const)
-            dir = RANGE_TO;   // Heuristic
-         else
-            dir = RANGE_DOWNTO;   // Heuristic
+         for (int i = 0; i < nranges; i++) {
+            type_t index_type = index_type_of(packed, i);
+            vlog_node_t vr = vlog_range(decl, i);
 
-         tree_t r = tree_new(T_RANGE);
-         tree_set_subkind(r, dir);
-         tree_set_left(r, left);
-         tree_set_right(r, right);
-         tree_set_loc(r, vlog_loc(decl));
-         tree_set_type(r, index_type);
+            tree_t left = trans_expr(g, vlog_left(vr));
+            tree_t right = trans_expr(g, vlog_right(vr));
 
-         tree_add_range(c, r);
+            int64_t ileft, iright;
+            const bool left_is_const = folded_int(left, &ileft);
+            const bool right_is_const = folded_int(right, &iright);
+
+            range_kind_t dir;
+            if (left_is_const && right_is_const)
+               dir = ileft < iright ? RANGE_TO : RANGE_DOWNTO;
+            else if (left_is_const)
+               dir = RANGE_TO;   // Heuristic
+            else
+               dir = RANGE_DOWNTO;   // Heuristic
+
+            tree_t r = tree_new(T_RANGE);
+            tree_set_subkind(r, dir);
+            tree_set_left(r, left);
+            tree_set_right(r, right);
+            tree_set_loc(r, vlog_loc(decl));
+            tree_set_type(r, index_type);
+
+            tree_add_range(c, r);
+         }
+
+         type_t sub = type_new(T_SUBTYPE);
+         type_set_base(sub, packed);
+         type_set_constraint(sub, c);
+
+         return sub;
       }
 
-      type_t sub = type_new(T_SUBTYPE);
-      type_set_base(sub, packed);
-      type_set_constraint(sub, c);
+   case DT_INT:
+   case DT_INTEGER:
+      return trans_sized_type(decl, packed_type, 32);
 
-      return sub;
+   case DT_BYTE:
+      return trans_sized_type(decl, packed_type, 8);
+
+   case DT_SHORTINT:
+      return trans_sized_type(decl, packed_type, 16);
+
+   case DT_LONGINT:
+      return trans_sized_type(decl, packed_type, 64);
+
+   default:
+      CANNOT_HANDLE(decl);
    }
-   else
-      return verilog_type(scalar_type);
 }
 
 static type_t trans_var_type(trans_gen_t *g, vlog_node_t v)
@@ -187,6 +257,7 @@ static void trans_port_decl(trans_gen_t *g, vlog_node_t v)
    tree_set_ident(t, vlog_ident(v));
    tree_set_subkind(t, map[kind]);
    tree_set_class(t, C_SIGNAL);
+   tree_set_loc(t, vlog_loc(v));
 
    vlog_node_t net = vlog_ref(v);
    if (vlog_is_net(net)) {
@@ -258,17 +329,25 @@ static void trans_net_decl(trans_gen_t *g, vlog_node_t decl)
    tree_add_decl(g->out, t);
 }
 
+static void trans_generic(trans_gen_t *g, vlog_node_t decl)
+{
+   tree_t wrap = tree_new(T_VERILOG);
+   tree_set_vlog(wrap, decl);
+   tree_set_ident(wrap, vlog_ident(decl));
+   tree_set_loc(wrap, vlog_loc(decl));
+
+   tree_add_decl(g->out, wrap);
+}
+
 void vlog_trans(vlog_node_t mod, tree_t out)
 {
-   assert(is_top_level(mod));
-
    trans_gen_t gen = {
       .map = hash_new(16),
       .out = out,
    };
 
    hset_t *ports = hset_new(16);
-   const int nports = vlog_ports(mod);
+   const int nports = vlog_kind(mod) == V_BLOCK ? 0 : vlog_ports(mod);
    for (int i = 0; i < nports; i++) {
       vlog_node_t ref = vlog_port(mod, i);
       assert(vlog_kind(ref) == V_REF);
@@ -285,12 +364,10 @@ void vlog_trans(vlog_node_t mod, tree_t out)
       case V_PORT_DECL:
          break;   // Translated below
       case V_VAR_DECL:
-         if (!hset_contains(ports, d))
-            trans_var_decl(&gen, d);
+         trans_var_decl(&gen, d);
          break;
       case V_NET_DECL:
-         if (!hset_contains(ports, d))
-            trans_net_decl(&gen, d);
+         trans_net_decl(&gen, d);
          break;
       case V_PARAM_DECL:
          trans_param_decl(&gen, d);
@@ -298,6 +375,13 @@ void vlog_trans(vlog_node_t mod, tree_t out)
       case V_LOCALPARAM:
          trans_localparam(&gen, d);
          break;
+      case V_FUNC_DECL:
+      case V_TASK_DECL:
+         trans_generic(&gen, d);
+         break;
+      case V_GENVAR_DECL:
+      case V_TYPE_DECL:
+         break;
       default:
          CANNOT_HANDLE(d);
       }
