@@ -46,6 +46,14 @@ static mir_value_t vlog_lower_rvalue(mir_unit_t *mu, vlog_node_t v)
    }
 }
 
+static mir_value_t vlog_udp_cmp(mir_unit_t *mu, mir_value_t left,
+                                mir_value_t right)
+{
+   mir_type_t t_logic = mir_vec4_type(mu, 1, false);
+   mir_value_t eq = mir_build_binary(mu, MIR_VEC_CASE_EQ, t_logic, left, right);
+   return mir_build_test(mu, eq);
+}
+
 void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
 {
    vlog_node_t udp = vlog_from_object(obj);
@@ -81,7 +89,6 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
 
    mir_type_t t_offset = mir_offset_type(mu);
    mir_type_t t_logic = mir_vec4_type(mu, 1, false);
-   mir_type_t t_time = mir_time_type(mu);
 
    mir_value_t result_var = mir_add_var(mu, t_logic, MIR_NULL_STAMP,
                                         ident_new("result"), MIR_VAR_TEMP);
@@ -110,9 +117,10 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
 
    mir_set_cursor(mu, start_bb, MIR_APPEND);
 
+   mir_build_store(mu, result_var, mir_const_vec(mu, t_logic, 1, 1));
+
    {
       mir_value_t one = mir_const(mu, t_offset, 1);
-      mir_value_t zero = mir_const(mu, t_time, 1);
       mir_value_t logic0 = mir_const_vec(mu, t_logic, 0, 0);
       mir_value_t logic1 = mir_const_vec(mu, t_logic, 1, 0);
       mir_value_t logicX = mir_const_vec(mu, t_logic, 1, 1);
@@ -154,18 +162,15 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
                case '1':
                case 'x':
                case 'X':
-                  cmp = mir_build_cmp(mu, MIR_CMP_EQ, in_regs[j],
-                                      level_map[vlog_ival(sym)]);
+                  cmp = vlog_udp_cmp(mu, in_regs[j], level_map[vlog_ival(sym)]);
                   break;
                case '*':
                   cmp = mir_build_event_flag(mu, in_nets[j], one);
                   break;
                case 'b':
                   {
-                     mir_value_t is0 =
-                        mir_build_cmp(mu, MIR_CMP_EQ, in_regs[j], logic0);
-                     mir_value_t is1 =
-                        mir_build_cmp(mu, MIR_CMP_EQ, in_regs[j], logic1);
+                     mir_value_t is0 = vlog_udp_cmp(mu, in_regs[j], logic0);
+                     mir_value_t is1 = vlog_udp_cmp(mu, in_regs[j], logic1);
                      cmp = mir_build_and(mu, is0, is1);
                   }
                   break;
@@ -188,14 +193,14 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
                         mir_build_last_value(mu, in_nets[j]);
                      mir_value_t last = mir_build_load(mu, last_ptr);
                      mir_value_t packed = mir_build_pack(mu, t_logic, last);
-                     mir_value_t eq = mir_build_cmp(mu, MIR_CMP_EQ, packed,
-                                                    level_map[(unsigned)left]);
+                     mir_value_t eq = vlog_udp_cmp(mu, packed,
+                                                   level_map[(unsigned)left]);
                      cmp = mir_build_and(mu, cmp, eq);
                   }
 
                   if (right != '?') {
-                     mir_value_t eq = mir_build_cmp(mu, MIR_CMP_EQ, in_regs[j],
-                                                    level_map[(unsigned)right]);
+                     mir_value_t eq = vlog_udp_cmp(mu, in_regs[j],
+                                                   level_map[(unsigned)right]);
                      cmp = mir_build_and(mu, cmp, eq);
                   }
                }
@@ -227,8 +232,7 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
             case '1':
             case 'x':
             case 'X':
-               cmp = mir_build_cmp(mu, MIR_CMP_EQ, packed,
-                                   level_map[vlog_ival(sym)]);
+               cmp = vlog_udp_cmp(mu, packed, level_map[vlog_ival(sym)]);
                break;
             case '?':
                break;
@@ -268,7 +272,7 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
          case '-':
             // No change, skip assignment to output
             drive = MIR_NULL_VALUE;
-            mir_build_wait(mu, start_bb, MIR_NULL_VALUE);
+            mir_build_wait(mu, start_bb);
             mir_set_cursor(mu, test_bb, MIR_APPEND);
             continue;
          default:
@@ -284,12 +288,8 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
       mir_set_cursor(mu, test_bb, MIR_APPEND);
 
       if (!mir_block_finished(mu, test_bb)) {
-         if (kind == V_UDP_SEQ)
-            mir_build_wait(mu, start_bb, MIR_NULL_VALUE);   // Skip assignment
-         else {
-            mir_build_store(mu, result_var, logicX);
-            mir_build_jump(mu, wait_bb);
-         }
+         mir_build_store(mu, result_var, logicX);
+         mir_build_jump(mu, wait_bb);
       }
 
       mir_set_cursor(mu, wait_bb, MIR_APPEND);
@@ -301,8 +301,8 @@ void vlog_lower_udp(mir_unit_t *mu, object_t *obj)
 
       mir_value_t upref = mir_build_var_upref(mu, hops, out_var.id);
       mir_value_t out = mir_build_load(mu, upref);
-      mir_build_sched_waveform(mu, out, one, drive, zero, zero);
+      mir_build_put_driver(mu, out, one, drive);
 
-      mir_build_wait(mu, start_bb, MIR_NULL_VALUE);
+      mir_build_wait(mu, start_bb);
    }
 }
