@@ -1,5 +1,5 @@
 //
-//  Copyright (C) 2023-2024  Nick Gasson
+//  Copyright (C) 2023-2025  Nick Gasson
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -477,6 +477,8 @@ START_TEST(test_vlog1)
 {
    input_from_file(TESTDIR "/dump/vlog1.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    vlog_node_t m1 = vlog_parse();
    fail_if(m1 == NULL);
 
@@ -508,6 +510,10 @@ START_TEST(test_vlog1)
              "  parameter logic [7:0] p1 = 5;\n"
              "  localparam bit p2 = 0;\n"
              "  logic [7:0] array1 [127:0];\n"
+             "  function func;\n"
+             "    input p;\n"
+             "  endfunction\n"
+             "  enum byte { a=5, b } e1;\n"
              "  initial begin\n"
              "    $display(\"hello\", $time);\n"
              "    if (bus) r <= 1 | r;\n"
@@ -521,7 +527,11 @@ START_TEST(test_vlog1)
              "    while (1);\n"
              "    repeat (5) r = r + 1;\n"
              "    for (i = 0; i < 10; i++) r = !r;\n"
+             "    #1;\n"
              "    forever i = ++i;\n"
+             "    r = func(4);\n"
+             "    i = ((bus >>> (1 * 4)) / ((r << 1) >> 2)) <<< 2;\n"
+             "    i += 1;\n"
              "  end\n"
              "  assign bus = 3;\n"
              "  pullup (supply0,supply1) g1 (w);\n"
