@@ -1943,6 +1943,23 @@ START_TEST(test_issue1318)
 }
 END_TEST
 
+START_TEST(test_issue1347)
+{
+   set_standard(STD_08);
+   input_from_file(TESTDIR "/simp/issue1347.vhd");
+
+   tree_t a = parse_check_and_simplify(T_ENTITY, T_ARCH);
+
+   tree_t c = tree_stmt(tree_stmt(a, 0), 0);
+   fail_unless(folded_i(tree_name(tree_assoc(tree_stmt(c, 0), 0)), 0));
+   fail_unless(folded_i(tree_name(tree_assoc(tree_stmt(c, 1), 0)), 42));
+   fail_unless(folded_i(tree_name(tree_assoc(tree_stmt(c, 2), 0)), 3));
+   fail_unless(folded_i(tree_name(tree_assoc(tree_stmt(c, 2), 1)), -1024));
+
+   fail_if_errors();
+}
+END_TEST
+
 Suite *get_simp_tests(void)
 {
    Suite *s = suite_create("simplify");
@@ -2018,6 +2035,7 @@ Suite *get_simp_tests(void)
    tcase_add_test(tc_core, test_issue1182);
    tcase_add_test(tc_core, test_issue1239);
    tcase_add_test(tc_core, test_issue1318);
+   tcase_add_test(tc_core, test_issue1347);
    suite_add_tcase(s, tc_core);
 
    return s;
