@@ -111,8 +111,8 @@ START_TEST(test_simple_sem)
 {
    const error_t expect[] = {
       {  7, "duplicate declaration of x" },
-      { 13, "no visible declaration for qq" },
-      { 13, "no visible declaration for dd" },
+      { 13, "no visible declaration for 'qq'" },
+      { 13, "no visible declaration for 'dd'" },
       { 19, "'r' cannot be driven by continuous assignment" },
       { 20, "'q' cannot be assigned in a procedural block" },
       { -1, NULL }
@@ -139,19 +139,22 @@ START_TEST(test_ports)
 {
    const error_t expect[] = {
       {  4, "duplicate declaration of y" },
-      { 11, "no visible declaration for y" },
+      { 11, "no visible declaration for 'y'" },
       { 19, "duplicate declaration of x" },
       { 22, "duplicate declaration of y" },
       { 31, "'o3' cannot be assigned in a procedural block" },
       { 43, "inconsistent dimensions for 'y'" },
       { 44, "cannot reference net 'x' in constant expression" },
+      { 47, "missing port declaration for 'zz'" },
+      { 55, "'o' does not appear in module port list" },
+      { 59, "'i1' does not appear in module port list" },
       { -1, NULL }
    };
    expect_errors(expect);
 
    input_from_file(TESTDIR "/vlog/ports.v");
 
-   for (int i = 0; i < 8; i++) {
+   for (int i = 0; i < 10; i++) {
       vlog_node_t m = vlog_parse();
       fail_if(m == NULL);
       fail_unless(vlog_kind(m) == V_MODULE);
@@ -169,12 +172,14 @@ START_TEST(test_parse1)
 {
    input_from_file(TESTDIR "/vlog/parse1.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    vlog_node_t m = vlog_parse();
    fail_if(m == NULL);
    fail_unless(vlog_kind(m) == V_MODULE);
-   ck_assert_int_eq(vlog_stmts(m), 33);
+   ck_assert_int_eq(vlog_stmts(m), 39);
    ck_assert_int_eq(vlog_ports(m), 0);
-   ck_assert_int_eq(vlog_decls(m), 28);
+   ck_assert_int_eq(vlog_decls(m), 36);
 
    vlog_node_t x = vlog_decl(m, 0);
    fail_unless(vlog_kind(x) == V_NET_DECL);
@@ -320,6 +325,10 @@ START_TEST(test_parse1)
    vlog_node_t et = vlog_stmt(s19b, 0);
    fail_unless(vlog_kind(et) == V_EVENT_TRIGGER);
 
+   vlog_node_t d28 = vlog_decl(m, 28);
+   fail_unless(vlog_kind(d28) == V_NET_DECL);
+   fail_unless(vlog_ident(d28) == ident_new("wire[x]net;"));
+
    fail_unless(vlog_parse() == NULL);
 
    fail_if_errors();
@@ -375,7 +384,7 @@ START_TEST(test_number2)
       { "1",             32,  1,           "1"              },
       { "42",            32,  42,          "42"             },
       { "251251",        32,  251251,      "251251"         },
-      { "'hffffFFff",    32,  0xffffffff,  "32'd4294967295" },
+      { "'hffffFFff",    32,  -1,          "-1"             },
       { "33'h100000000", 33,  0x100000000,
         "33'b100000000000000000000000000000000"             },
       { "64'b0",         64,  0,           "64'b0"          },
@@ -385,11 +394,14 @@ START_TEST(test_number2)
       { "4'h12",         4,   2,           "4'd2"           },
       { "1'b110",        1,   0,           "1'b0"           },
       { "1_000",         32,  1000,        "1000"           },
+      { "6'o02",         6,   2,           "6'd2"           },
+      { "4'o_1",         4,   1,           "4'd1",          }
    };
 
    const error_t expect[] = {
       { LINE_INVALID, "excess digits in hex constant 4'h12" },
       { LINE_INVALID, "excess digits in binary constant 1'b110" },
+      { LINE_INVALID, "number cannot start with an underscore" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -435,15 +447,20 @@ START_TEST(test_pp1)
       "\n"
       "pass\n"
       "\n"
-      TESTDIR "/vlog/pp1.v\n"
-      "16\n"
+      "\""TESTDIR "/vlog/pp1.v""\"\n"
+      "\"16\"\n"
+      "\n"
+      "\n"
+      "\n"
+      "\n"
+      "\n"
       "\n");
 }
 END_TEST
 
 START_TEST(test_empty1)
 {
-   input_from_buffer("", 0, SOURCE_VERILOG);
+   input_from_buffer("", 0, FILE_INVALID, SOURCE_VERILOG);
 
    fail_unless(vlog_parse() == NULL);
 
@@ -460,6 +477,8 @@ START_TEST(test_timescale1)
    const error_t expect[] = {
       {  2, "invalid time unit name 'hello'" },
       {  3, "invalid order of magnitude in `timescale directive" },
+      {  4, "unexpected real while parsing timescale compiler directive, "
+         "expecting unsigned number" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -505,9 +524,12 @@ START_TEST(test_pp2)
    const error_t expect[] = {
       {  1, "`else outside of `ifdef" },
       {  2, "`endif outside of `ifdef" },
-      {  3, "expected macro name after `ifdef" },
-      {  4, "expected macro name after `ifndef" },
-      {  5, "expected macro name after `define" },
+      {  3, "`elsif outside of `ifdef" },
+      {  3, "expected macro name after `elsif" },
+      {  4, "expected macro name after `ifdef" },
+      {  5, "expected macro name after `ifndef" },
+      {  7, "expected macro name after `elsif" },
+      {  9, "no corresponding `endif before end of file" },
       {  6, "no corresponding `endif before end of file" },
       { -1, NULL }
    };
@@ -541,11 +563,12 @@ START_TEST(test_udp1)
    input_from_file(TESTDIR "/vlog/udp1.v");
 
    const error_t expect[] = {
-      { 13, "no visible declaration for z" },
+      { 13, "no visible declaration for 'z'" },
       { 13, "the first port of a primitive must be an output" },
       { 13, "all ports of a primitive except the first must be inputs" },
       { 25, "expected 3 symbols in UDP table entry but have 2" },
       { 58, "a sequential input list may have at most one edge indicator" },
+      { 63, "unexpected identifier while parsing integral number, " },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -584,10 +607,13 @@ END_TEST
 
 START_TEST(test_struct1)
 {
-   input_from_file(TESTDIR "/vlog/struct1.v");
+   input_from_file(TESTDIR "/vlog/struct1.sv");
 
    const error_t expect[] = {
       { 13, "duplicate declaration of a" },
+      { 18, "struct has no field named 'c'" },
+      { 19, "prefix is not a struct or class" },
+      { 20, "'t_pair' cannot be assigned in a procedural block" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -608,10 +634,11 @@ END_TEST
 
 START_TEST(test_enum1)
 {
-   input_from_file(TESTDIR "/vlog/enum1.v");
+   input_from_file(TESTDIR "/vlog/enum1.sv");
 
    const error_t expect[] = {
       {  5, "'a' is not a data type" },
+      {  8, "cannot reference variable 'zz' in constant expression" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -622,7 +649,7 @@ START_TEST(test_enum1)
 
    vlog_check(m);
 
-   ck_assert_int_eq(vlog_decls(m), 4);
+   ck_assert_int_eq(vlog_decls(m), 8);
 
    fail_unless(vlog_parse() == NULL);
 
@@ -632,7 +659,7 @@ END_TEST
 
 START_TEST(test_union1)
 {
-   input_from_file(TESTDIR "/vlog/union1.v");
+   input_from_file(TESTDIR "/vlog/union1.sv");
 
    vlog_node_t m = vlog_parse();
    fail_if(m == NULL);
@@ -652,6 +679,8 @@ START_TEST(test_param1)
 {
    input_from_file(TESTDIR "/vlog/param1.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    const error_t expect[] = {
       {  4, "duplicate declaration of p1" },
       { -1, NULL }
@@ -674,6 +703,8 @@ START_TEST(test_param2)
 {
    input_from_file(TESTDIR "/vlog/param2.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    const error_t expect[] = {
       { 13, "duplicate declaration" },
       { 18, "local parameter declaration must have a default value" },
@@ -696,6 +727,24 @@ START_TEST(test_param2)
 
    vlog_check(m2);
 
+   vlog_node_t m3 = vlog_parse();
+   fail_if(m3 == NULL);
+   fail_unless(vlog_kind(m3) == V_MODULE);
+
+   vlog_node_t p3 = vlog_decl(m3, 0);
+   fail_unless(vlog_kind(p3) == V_LOCALPARAM);
+   vlog_node_t p3v = vlog_value(p3);
+   fail_unless(vlog_kind(p3v) == V_MIN_TYP_MAX);
+
+   vlog_node_t min = vlog_left(p3v);
+   fail_unless(number_integer(vlog_number(min)) == 5);
+
+   vlog_node_t typ = vlog_value(p3v);
+   fail_unless(number_integer(vlog_number(typ)) == 6);
+
+   vlog_node_t max = vlog_right(p3v);
+   fail_unless(number_integer(vlog_number(max)) == 7);
+
    fail_unless(vlog_parse() == NULL);
 
    check_expected_errors();
@@ -707,7 +756,7 @@ START_TEST(test_pp3)
    input_from_file(TESTDIR "/vlog/pp3.v");
 
    const error_t expect[] = {
-      {  3, "macro FOO undefined" },
+      {  3, "macro 'FOO' undefined" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -731,6 +780,7 @@ START_TEST(test_concat1)
 
    const error_t expect[] = {
       { 10, "'q' cannot be driven by continuous assignment" },
+      { 12, "cannot reference net 'z' in constant expression" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -752,11 +802,11 @@ START_TEST(test_pp4)
    input_from_file(TESTDIR "/vlog/pp4.v");
 
    const error_t expect[] = {
-      { 11, "unexpected module while parsing expression" },
+      { 16, "unexpected module while parsing expression" },
       {  0, "this token was unexpected" },
       {  0, "while expanding macro MACRO2" },
       {  0, "while expanding macro MACRO1" },
-      { 16, "unexpected wire while parsing net declaration assignment" },
+      { 21, "unexpected wire while parsing net declaration assignment" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -764,7 +814,9 @@ START_TEST(test_pp4)
    LOCAL_TEXT_BUF tb = tb_new();
    vlog_preprocess(tb, true);
 
-   input_from_buffer(tb_get(tb), tb_len(tb), SOURCE_VERILOG);
+   file_ref_t file_ref = loc_file_ref(TESTDIR "/vlog/pp4.v", NULL);
+
+   input_from_buffer(tb_get(tb), tb_len(tb), file_ref, SOURCE_VERILOG);
 
    vlog_node_t m = vlog_parse();
    fail_if(m == NULL);
@@ -809,6 +861,8 @@ START_TEST(test_const1)
 {
    input_from_file(TESTDIR "/vlog/const1.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    const error_t expect[] = {
       {  4, "cannot reference net 'w1' in constant expression" },
       { -1, NULL }
@@ -832,7 +886,7 @@ START_TEST(test_case1)
    input_from_file(TESTDIR "/vlog/case1.v");
 
    const error_t expect[] = {
-      { 10, "no visible declaration for aaa" },
+      { 10, "no visible declaration for 'aaa'" },
       { 15, "multiple default statements within a single case statement" },
       { -1, NULL }
    };
@@ -854,10 +908,14 @@ START_TEST(test_direct1)
 {
    input_from_file(TESTDIR "/vlog/direct1.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    const error_t expect[] = {
       { 11, "unexpected identifier while parsing default_nettype directive, "
         "expecting one of wire, tri, tri0, tri1, wand, triand, wor, trior," },
-      { 15, "no visible declaration for x" },
+      { 15, "no visible declaration for 'x'" },
+      { 29, "unexpected wire while parsing unconnected_drive directive, "
+        "expecting one of pull0 or pull1" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -886,12 +944,13 @@ START_TEST(test_string1)
       const char *input;
       const char *output;
    } cases[] = {
-      { "\"hello\"", "hello" },
-      { "\"\"", "" },
+      { "hello", "hello" },
+      { "", "" },
+      { "1\0002", "1\0002" },
    };
 
    for (int i = 0; i < ARRAY_LEN(cases); i++) {
-      number_t n = number_new(cases[i].input, NULL);
+      number_t n = number_from_string(cases[i].input, strlen(cases[i].input));
       ck_assert(number_is_defined(n));
 
       const int width = number_width(n);
@@ -931,14 +990,17 @@ END_TEST
 
 START_TEST(test_tfcall1)
 {
-   input_from_file(TESTDIR "/vlog/tfcall1.v");
+   input_from_file(TESTDIR "/vlog/tfcall1.sv");
 
    const error_t expect[] = {
+      { 43, "return statement can only be used in a subroutine" },
+      { 40, "return statement in a task cannot have an expression" },
+      { 46, "return statement in a non-void function must have an expression" },
       {  8, "expected 2 arguments for 'sum' but have 1" },
       {  9, "expected 2 arguments for 'sum' but have 3" },
       { 13, "'x4' is not a function" },
       { 27, "'no_args' is not a task" },
-      { 12, "no visible declaration for not_here" },
+      { 12, "no visible declaration for 'not_here'" },
       { -1, NULL }
    };
    expect_errors(expect);
@@ -959,6 +1021,8 @@ START_TEST(test_attr1)
 {
    input_from_file(TESTDIR "/vlog/attr1.v");
 
+   set_default_keywords(VLOG_1800_2023);
+
    const error_t expect[] = {
       { 11, "attribute instance is not allowed here" },
       { -1, NULL }
@@ -1030,6 +1094,178 @@ START_TEST(test_nets1)
 }
 END_TEST
 
+START_TEST(test_keywords)
+{
+   input_from_file(TESTDIR "/vlog/keywords.v");
+
+   const error_t expect[] = {
+      {  9, "'logic' is a keyword in System Verilog 1800-2005" },
+      { 13, "`end_keywords directive without matching `begin_keywords" },
+      { 14, "\"fizz\" is not a recognised Verilog or System Verilog version" },
+      { 15, "unexpected unsigned number while parsing keywords directive, "
+        "expecting string" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   for (int i = 0; i < 2; i++) {
+      vlog_node_t m = vlog_parse();
+      fail_if(m == NULL);
+      fail_unless(vlog_kind(m) == V_MODULE);
+   }
+
+   fail_unless(vlog_parse() == NULL);
+
+   check_expected_errors();
+}
+END_TEST
+
+START_TEST(test_error1)
+{
+   input_from_file(TESTDIR "/vlog/error1.v");
+
+   const error_t expect[] = {
+      {  2, "no data type declaration for 'foo'" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   vlog_node_t m = vlog_parse();
+   fail_if(m == NULL);
+   fail_unless(vlog_kind(m) == V_MODULE);
+
+   fail_unless(vlog_parse() == NULL);
+
+   check_expected_errors();
+}
+END_TEST
+
+START_TEST(test_pp5)
+{
+   input_from_file(TESTDIR "/vlog/pp5.v");
+
+   LOCAL_TEXT_BUF tb = tb_new();
+   vlog_preprocess(tb, false);
+
+   ck_assert_str_eq(tb_get(tb), "\n\n\n\n\n\n\n\n\n\n //  `ifdef VERIFIC\n");
+}
+END_TEST
+
+START_TEST(test_integers1)
+{
+   input_from_file(TESTDIR "/vlog/integers1.sv");
+
+   const error_t expect[] = {
+      {  21, "unexpected identifier while parsing statement item, expecting ;" },
+      {  28, "unexpected ' while parsing statement item, expecting ;" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   vlog_node_t m = vlog_parse();
+   fail_if(m == NULL);
+   fail_unless(vlog_kind(m) == V_MODULE);
+
+   fail_unless(vlog_parse() == NULL);
+
+   check_expected_errors();
+}
+END_TEST
+
+START_TEST(test_pp6)
+{
+   add_include_dir(TESTDIR "/vlog");
+
+   input_from_file(TESTDIR "/vlog/pp6.v");
+
+   const error_t expect[] = {
+      {  2, "macro 'bad' undefined" },
+      {  4, "only white space or a comment may appear on the same "
+         "line as an `include directive" },
+      {  5, "cannot find not.here in the current working directory" },
+      {  0, "searched include directory " TESTDIR "/vlog" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   LOCAL_TEXT_BUF tb = tb_new();
+   vlog_preprocess(tb, false);
+
+   ck_assert_str_eq(
+      tb_get(tb),
+      "one\n"
+      "hello\n"
+      "\n"
+      "two\n"
+      "23\n");
+
+   check_expected_errors();
+}
+END_TEST
+
+START_TEST(test_label1)
+{
+   input_from_file(TESTDIR "/vlog/label1.v");
+
+   const error_t expect[] = {
+      { 10, "cannot specify both a statement label and a block name" },
+      { 20, "cannot specify both a statement label and a block name" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   vlog_node_t m = vlog_parse();
+   fail_if(m == NULL);
+   fail_unless(vlog_kind(m) == V_MODULE);
+
+   fail_unless(vlog_parse() == NULL);
+
+   check_expected_errors();
+}
+END_TEST
+
+START_TEST(test_href1)
+{
+   input_from_file(TESTDIR "/vlog/href1.v");
+
+   const error_t expect[] = {
+      {  4, "prefix of hierarchical identifier is not an instance" },
+      {  5, "no visible declaration for 'xx'" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   vlog_node_t m = vlog_parse();
+   fail_if(m == NULL);
+   fail_unless(vlog_kind(m) == V_MODULE);
+
+   vlog_check(m);
+
+   fail_unless(vlog_parse() == NULL);
+
+   check_expected_errors();
+}
+END_TEST
+
+START_TEST(test_package1)
+{
+   input_from_file(TESTDIR "/vlog/package1.sv");
+
+   const error_t expect[] = {
+      {  3, "'different_name' does not match package name 'dummy_pkg'" },
+      { -1, NULL }
+   };
+   expect_errors(expect);
+
+   vlog_node_t m = vlog_parse();
+   fail_if(m == NULL);
+   fail_unless(vlog_kind(m) == V_PACKAGE);
+
+   check_expected_errors();
+}
+END_TEST
+
+
 Suite *get_vlog_tests(void)
 {
    Suite *s = suite_create("vlog");
@@ -1067,6 +1303,14 @@ Suite *get_vlog_tests(void)
    tcase_add_test(tc, test_attr1);
    tcase_add_test(tc, test_initial1);
    tcase_add_test(tc, test_nets1);
+   tcase_add_test(tc, test_keywords);
+   tcase_add_test(tc, test_error1);
+   tcase_add_test(tc, test_pp5);
+   tcase_add_test(tc, test_integers1);
+   tcase_add_test(tc, test_pp6);
+   tcase_add_test(tc, test_label1);
+   tcase_add_test(tc, test_href1);
+   tcase_add_test(tc, test_package1);
    suite_add_tcase(s, tc);
 
    return s;
