@@ -54,6 +54,7 @@ void vhpi12_startup(void);
 void vhpi13_startup(void);
 void vhpi14_startup(void);
 void vhpi15_startup(void);
+void vhpi16_startup(void);
 void issue744_startup(void);
 void issue762_startup(void);
 void issue978_startup(void);
@@ -65,5 +66,7 @@ void issue1060_startup(void);
 void issue1161_startup(void);
 void issue1192_startup(void);
 void issue1213_startup(void);
+void issue1233_startup(void);
+void issue1240_startup(void);
 
 #endif  // _VHPI_TEST_H
